/* $Id: e2_complete__path.c,v 1.5 2004/08/14 12:01:08 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <glob.h>
#include "emelfm2.h"
#include "e2_utf8.h"
#include "e2_complete.h"

static gchar **get_path_dirs ()
{
	const gchar *p = getenv ("PATH");
	gchar **ret = g_strsplit (p, ":", -1);
	return ret;
}

static gint check_dir (gchar *dir, gchar *word, GList **found, gchar *prefix)
{
	printd (DEBUG, "check_dir (dir:%s,word:%s,found:_,prefix:%s)", dir, word, prefix);
	GError *error = NULL;
	GDir *d = g_dir_open (dir, 0, &error);
	if (error != NULL)
		return 0;
	gint ret = 0;
	const gchar *cur = g_dir_read_name (d);
	while (cur != NULL)
	{
		if (g_str_has_prefix (cur, word))
		{
			gchar *tmp = g_strconcat (dir, "/", cur, NULL);
			gboolean is_exec = g_file_test (tmp, G_FILE_TEST_IS_EXECUTABLE);
			gboolean is_dir = g_file_test (tmp, G_FILE_TEST_IS_DIR);
			if (is_exec || is_dir)
			{
				printd (DEBUG, "match: %s - %s", cur, word);
				gchar *utf = _STR2UTF (cur);
				if (is_dir)
				{
					gchar *free = utf;
					utf = g_strconcat (utf, "/", NULL);
					g_free (free);
				}
				if (prefix == NULL)
					*found = g_list_append (*found, utf);
				else
				{
					gchar *f = g_strconcat (prefix, utf, NULL);
					*found = g_list_append (*found, f);
					g_free (utf);
				}
				ret++;
			}
			g_free (tmp);
		}
		cur = g_dir_read_name (d);
	}
	
	g_dir_close (d);
	return ret;
}

gint e2_complete_path (gchar *line, gchar *word, gint pos, GList **found,
	E2_CompleteFlags *flags, gpointer data)
{
	printd (DEBUG, "complete_path (line:%s,word:%s,pos:%d,found:_,flags:%d,data:_)",
		line, word, pos, *flags);

	//we shall return the number of successful matches
	gint ret = 0;

	if ((word == NULL) || (*word == '\0'))
		return ret;
	
	//check if the word we shall complete is the only one on the line
	if (!g_str_equal (word, line))
		return ret;
	
	gchar *word_locale = _UTF2STR (word);
	if (g_str_has_prefix (word, "./"))
	{
		*flags |= E2_COMPLETE_FLAG_STOP;
		gchar *p1 = g_utf8_next_char (g_utf8_strrchr (word, -1, '/'));
		gchar *p2 = e2_utf8_ndup (word, g_utf8_pointer_to_offset (word, p1));
		gchar *path = g_strconcat (app.pane_current->path, "/", p2, NULL);
		ret += check_dir (path, p1, found, p2);
		g_free (p2);
		g_free (path);
	} else
	{
		printd (DEBUG, "complete from path");
		gchar **paths = get_path_dirs ();
		gint i = 0;
		while (paths[i])
		{
			ret += check_dir (paths[i], word_locale, found, NULL);
				i++;
		}
		g_strfreev (paths);
	}
	g_free (word_locale);

	if (ret > 0)
		*flags |= E2_COMPLETE_FLAG_STOP;
	return ret;
}

