/* $Id: e2_command.c,v 1.25 2004/06/29 15:24:19 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <sys/wait.h>

#include "emelfm2.h"
#include "e2_command.h"
#include "e2_command_line.h"
#include "e2_complete.h"
#include "e2_output.h"
#include "e2_action.h"
#include "e2_utils.h"
#include "e2_alias.h"
#include "debug.h"

GList *children = NULL;
static void sigchld_handler (int num);
static E2_CommandRuntime *register_child (gint pid, gint mode, gchar *command, gint exit, GIOChannel *in);

static void block ()
{
	sigset_t set, oldset;

	//block other incoming SIGCHLD signals
	sigemptyset (&set);
	sigaddset (&set, SIGCHLD);
	sigprocmask (SIG_BLOCK, &set, &oldset);
}

static void unblock ()
{
	sigset_t set, oldset;
	
	//re-install the signal handler (some systems need this)
	signal (SIGCHLD, sigchld_handler);
	
	//and unblock it
	sigemptyset (&set);
	sigaddset (&set, SIGCHLD);
	sigprocmask (SIG_UNBLOCK, &set, &oldset);
}
/*
static void signal_handler (gint sig)
{
	printd (DEBUG, "signal: %d", sig);
}
*/

//copied from gtk faq @ http://gtk.org
static void sigchld_handler (int num)
{
	printd (DEBUG, "sigchld_handler (num:%d)", num);
	pid_t pid;
	int status, exitstatus;

	block ();

	//wait for child
	while ((pid = waitpid ((pid_t) -1, &status, WNOHANG)) > 0)
	{
		if(WIFEXITED(status))
		{
			exitstatus = WEXITSTATUS (status);
			printd (DEBUG, "child (%d) exit status = %d", (gint) pid,
				exitstatus);
		}
		else if (WIFSIGNALED (status))
		{
			exitstatus = WTERMSIG (status);
			printd (DEBUG, "child (%d) terminated by signal %d",
				(gint) pid, exitstatus);
		} else if (WIFSTOPPED (status))
		{
			exitstatus = WSTOPSIG (status);
			printd (DEBUG, "child (%d) stopped by signal %d", (gint) pid, exitstatus);
		} else
			printd (DEBUG, "child (%d) exitted magically", (gint) pid);
		
		if (pid > 0)
		{
			E2_CommandRuntime *rt;
			while ((rt = register_child (pid, 0, NULL, status, NULL)) == NULL);
		}
		
	}

	unblock ();
}

static void set_sigchild_handler ()
{
	signal(SIGCHLD, sigchld_handler);
	/*
	signal(SIGPOLL, signal_handler);
	signal(SIGABRT,signal_handler);
	signal(SIGALRM,signal_handler);
	signal(SIGHUP,signal_handler);
	signal(SIGINT,signal_handler);
	signal(SIGPIPE,signal_handler);
	signal(SIGQUIT,signal_handler);
	signal(SIGTERM,signal_handler);
	signal(SIGUSR1,signal_handler);
	signal(SIGUSR2,signal_handler);
	*/
}

static gboolean watch (GIOChannel *ioc, GIOCondition cond,
	E2_CommandRuntime *rt, gboolean error);

static gboolean watch_std (GIOChannel *ioc, GIOCondition cond,
	E2_CommandRuntime *rt)
{
	return watch (ioc, cond, rt, FALSE);
}
static gboolean watch_err (GIOChannel *ioc, GIOCondition cond,
	E2_CommandRuntime *rt)
{
	return watch (ioc, cond, rt, TRUE);
}

static gboolean watch (GIOChannel *ioc, GIOCondition cond,
	E2_CommandRuntime *rt, gboolean error)
{
//	printd (DEBUG, "watch (ioc:,cond:%d,rt:)", cond);
	if (cond & (G_IO_IN | G_IO_PRI))
	{
		gchar buf[4096];
		gsize len = 0;
		GIOStatus ret = g_io_channel_read_chars (ioc, buf,
			sizeof (buf) - 1, &len, NULL);
		buf[len] = '\0';
		if (!(len < 1 || (ret & G_IO_STATUS_ERROR) || (ret & G_IO_STATUS_EOF)))
		{
			if (rt->show || error)
			{
				gchar save = 0;
				gchar *end;
				gchar *print = buf;
				while (TRUE)
				{
					gboolean next = FALSE;
					if ((end = strchr (print, '\n')) != NULL)
					{
						end++;
						save = *end;
						*end = '\0';
						next = TRUE;
					}
					e2_output_print (print, rt->pidstr, error, FALSE, FALSE, NULL);

					if (next)
					{
						*end = save;
						print = end;
						end = NULL;
					} else
						break;
				}
			}
			if (ret & G_IO_STATUS_NORMAL)
				return TRUE;
		}
	}
	if (rt->closed && (cond & (G_IO_ERR | G_IO_HUP | G_IO_NVAL)))
	{
		block ();
		if (rt->show)
		{
			gchar *message = g_strdup_printf (">%s (%d) returned '%d'",
				rt->command, rt->pid, WEXITSTATUS (rt->exit));
			e2_output_print (message, rt->pidstr, FALSE, FALSE, TRUE, "small",
				"grey", NULL);
			g_free (message);
			E2_CommandRuntime *rt2;
			while ((rt2 = register_child
					(rt->pid, 2, NULL, 0, NULL)) == NULL);
		}
		g_free (rt->command);
		g_free (rt->pidstr);
		g_io_channel_shutdown (rt->ioin, TRUE, NULL);
		g_io_channel_unref (rt->ioin);
		g_free (rt);
			unblock ();
		return FALSE;
	}
	if (!rt->closed && (cond & (G_IO_ERR | G_IO_HUP | G_IO_NVAL)))
	{
		rt->closed = TRUE;
		g_io_channel_shutdown (ioc, TRUE, NULL);
	}
	return FALSE;
}

static GIOChannel *set_up_io_channel (gint fd, GIOCondition cond, gpointer func,
	gpointer data)
{

	// set up handler for data
	GIOChannel *ioc = g_io_channel_unix_new (fd);

	// Set IOChannel encoding to none to make it fit for binary data
	g_io_channel_set_encoding (ioc, NULL, NULL);
	g_io_channel_set_buffered (ioc, FALSE);
	g_io_channel_set_close_on_unref (ioc, TRUE);

	g_io_add_watch_full (ioc, e2_option_int_get ("command-watch-priority"),
		cond, (GIOFunc) func, data, NULL);

	// g_io_add_watch () adds reference
	g_io_channel_unref (ioc);
	return ioc;
}

static E2_CommandRuntime *find_child (gint pid)
{
	if (children != NULL)
	{
		GList *tmp = g_list_first (children);
		E2_CommandRuntime *rt;
		while ((tmp != NULL))
		{
			rt = tmp->data;
			if ((rt != NULL) && (rt->pid == pid))
				return rt;
			tmp = g_list_next (tmp);
		}
	}
	return NULL;
}

static E2_CommandRuntime *register_child (gint pid, gint mode, gchar *command, gint exit, GIOChannel *in)
{
	static gboolean blocked = FALSE;
	if (blocked)
		return NULL;
	else
		blocked = TRUE;
	printd (DEBUG, "register_child (pid:%d,mode:%d,command:%s,exit:%d,in:)", pid,
		mode, command, exit);
	E2_CommandRuntime *rt = find_child (pid);
	if (rt != NULL)
	{
		if (mode == 1)
		{
			rt->start = TRUE;
			rt->command = g_strdup (command);
			rt->ioin = in;
		} else if (mode == 0)
		{
			rt->end = TRUE;
			rt->exit = exit;
		}
		if ((mode == 2) || ((mode == 0) && (!rt->show)))
		{
			if (g_list_length (children) < 2)
			{
				g_list_free (children);
				children = NULL;
			} else
				children = g_list_remove (children, rt);
		}
	} else
	{
		rt = malloc (sizeof (E2_CommandRuntime));
		if (mode == 1)
		{
			rt->start = TRUE;
			rt->end = FALSE;
			rt->command = g_strdup (command);
			rt->ioin = in;
		} else if (mode == 0)
		{
			rt->start = FALSE;
			rt->end = TRUE;
			rt->exit = exit;
		} else if (mode == 2)
		{
			printd (ERROR, "program bug in command handling, aborting");
			quit_cb (NULL);
		}
		rt->pid = pid;
		rt->pidstr = g_strdup_printf ("%d", rt->pid);
		
		children = g_list_append (children, rt);
	}
	blocked = FALSE;
	return rt;
}

void e2_command_list_children (gpointer data)
{
	printd (DEBUG, "e2_command_list_children (data:)");

	block ();
	
	if (children == NULL)
	{
		status_message ("no children\n");
		return;
	}
	GList *tmp = g_list_first (children);
	gint i = 0;
	gchar *message = g_strdup ("| id |  pid  | command\n|----|-------|-----------------\n");
	status_message (message);
	g_free (message);
	while (tmp != NULL)
	{
		E2_CommandRuntime *rt = tmp->data;
		
		if ((rt != NULL) && (rt->start))
		{
			i++;
			message = g_strdup_printf ("| %2d | %5d | %s\n",
				i, rt->pid, rt->command);
			status_message (message);
			g_free (message);
		}
		tmp = g_list_next (tmp);
	}
	
	status_message ("\n");

	unblock ();
}

gboolean e2_command_send_to_pid (gint pid, gchar *command)
{
	printd (DEBUG, "e2_command_send_to_pid (pid:%d,command:%s)", pid, command);
	E2_CommandRuntime *rt;
	if (pid == 0)
	{
		GList *end = g_list_last (children);
		if (end == NULL)
			return FALSE;
		rt = end->data;
	} else
		rt = find_child (pid);
	if ((rt == NULL) || (!rt->start))
	{
		gchar *message = g_strdup_printf ("The process with pid %d is not a child of us.", pid);
		e2_output_print (message, NULL, TRUE, FALSE, TRUE, NULL);
		g_free (message);
		return FALSE;
	}
	gsize size;
	GError *error = NULL;
	GIOStatus stat = g_io_channel_write_chars (rt->ioin, command, -1, &size, &error);
	if (error != NULL)
	{
		gchar *message = g_strconcat ("Failed writing to child: ", error->message, ".", NULL);
		printd (WARN, message);
		e2_output_print (message, rt->pidstr, TRUE, FALSE, TRUE, NULL);
		g_error_free (error);
		g_free (message);
		return FALSE;
	}
	error = NULL;
	stat = g_io_channel_write_chars (rt->ioin, "\n", 1, &size, &error);
	if (error != NULL)
	{
		gchar *message = g_strconcat ("Failed writing to child: ", error->message, ".", NULL);
		printd (WARN, message);
		e2_output_print (message, rt->pidstr, TRUE, FALSE, TRUE, NULL);
		g_error_free (error);
		g_free (message);
		return FALSE;
	} else
	{
		gchar *message = g_strconcat (command, "\n", NULL);
		e2_output_print (message, rt->pidstr, FALSE, FALSE, FALSE, NULL);
		g_free (message);
	}

	return TRUE;
}

static gint run_checks (gchar *command, gboolean *macros, gboolean *shell,
	gboolean *show)
{
	gint retval = 0;
	while (TRUE)
	{
		if (command[0] == '!')
		{
			retval++;
			command++;
			*macros = FALSE;
			continue;
		}
		if (command[0] == '%')
		{
			retval++;
			command++;
			*shell = TRUE;
			continue;
		}
		break;
	}
	if (g_str_has_suffix (command, "&"))
	{
		*show = FALSE;
		command[strlen (command) - 1] = '\0';
	}
	return retval;
}

gint e2_command_run (gchar *command, E2_CommandRange range)
{
	if ((command == NULL) || (*command == '\0'))
		return 0;
	g_strstrip (command);
	if (*command == '\0')
		return 0;
	
	printd (DEBUG, "e2_command_run (command:%s,range:%d)", command, range);

	if ((command[0] == ';') && ((strchr (command + 1, ';')) != NULL))
	{
		gchar **spt = g_strsplit (command, ";", -1);
		gint i = 0;
		while (spt[i])
		{
			e2_command_run (spt[i++], range);
		}
		g_strfreev (spt);
		return 0;
	}

	gboolean shall_expand_macros = TRUE;
	gboolean run_in_shell = FALSE;
	gboolean show = TRUE;
	gboolean wait = FALSE;
	
	command += run_checks (command, &shall_expand_macros, &run_in_shell, &show);

	gboolean free = FALSE;
	if ((range == E2_COMMAND_RANGE_FILE_ACTION) &&
		(strchr (command, '%') == NULL) && shall_expand_macros)
	{
		GString *str = g_string_new (command);
		GList *tmp, *base;
		base = tmp = get_selection (curr_view);
		if (tmp != NULL)
		{
			for (; tmp != NULL; tmp = tmp->next)
			{
				FileInfo *info = tmp->data;
				g_string_append_printf (str, " \"%s\"", info->filename);
			}
			g_list_free (base);
		}
		command = str->str;
		g_string_free (str, FALSE);
//		shall_expand_macros = FALSE;
		free = TRUE;
	}

	gchar *command_new;
	if ((range != E2_COMMAND_RANGE_FILE_OPERATION) && shall_expand_macros &&
		e2_option_bool_get ("command-use-aliases"))
		command_new = e2_alias_apply (command, &app.aliases);
	else
		command_new = g_strdup (command);

	if (free)
		g_free (command);
	gint ret = run_checks (command_new, &shall_expand_macros, &run_in_shell, &show);
	command = e2_utils_replace_env_vars (command_new + ret);
	g_free (command_new);
	
	if (range == E2_COMMAND_RANGE_FILE_OPERATION)
	{
		wait = !e2_option_bool_get ("fileop-background");
		show = e2_option_bool_get ("fileop-show");
	}

	//really expand macros
	if (shall_expand_macros)
	{
		GString *str = NULL;
		if ((str = expand_macros (command, NULL)) == NULL)
		{
			status_error ("failed expanding macros");
			return 0;
		}
		g_free (command);
		command = str->str;
		g_string_free (str, FALSE);
	}
	
	gchar *command_real = NULL;
	if ((range != E2_COMMAND_RANGE_FILE_OPERATION) &&
		((command_real = strchr (command, ':')) != NULL))
	{
		gint dif = command_real - command;
		if (dif < 10)
		{
			gchar command_save = *command_real;
			*command_real = '\0';
			command_real++;
			gchar *end = NULL;
			gint i = g_ascii_strtoull (command, &end, 10);
			if ((end != command) || (dif == 0))
			{
				printd (DEBUG, "sending '%s' to process '%s'", command_real, command);
				e2_command_send_to_pid (i, command_real);
				return 0;
			}
			command_real--;
			*command_real = command_save;
		}
	}

	if (!run_in_shell && (g_str_has_prefix (command, "cd")))
	{
		gint len = strlen (command);
		if ((len > 3) && (command[2] == ' '))
		if ((strlen (command) > 3) && (command[2] == ' '))
		{
			gchar *utf = _STR2UTF (command + 3);
			e2_pane_change_dir (NULL, utf);
			g_free (utf);
			return 0;
		}
		if (len == 2)
		{
			gchar *home = (gchar *) g_get_home_dir ();
			e2_pane_change_dir (NULL, home);
			return 0;
		}
	} else if (g_str_has_prefix (command, "?"))
	{
		if ((strlen (command)) == 1)
		{
			e2_output_print (_("You need to specify an action to run like '?action'.\n"),
				NULL, TRUE, FALSE, FALSE, NULL);
			g_free (command);
			return 0;
		}
		//FIXME: do not require a comma ',' as argument seperator
		gchar *args = strchr (command + 1, ',');
		if (args != NULL)
			*args = '\0';
		e2_action_run_simple (command + 1, (args != NULL ? args + 1 : NULL));
		return 0;
	}

	gint tmp;
	gchar **args;
	free = TRUE;
	if (run_in_shell)
	{
		free = FALSE;
		gchar *args_[4] = { "/bin/sh", "-c", command };
		args = args_;
		tmp = 3;
	} else
	{
		GError *error = NULL;
		g_shell_parse_argv (command, &tmp, &args, &error);
		if (error != NULL)
		{
			gchar *error_message = g_strconcat ("Failed parsing command line '",
				command, "':\n", NULL);
			e2_output_print_error (error_message);
			g_free (error_message);
			status_error (error->message);
			g_error_free (error);
			g_free (command);
			return 0;
		}
	}

	gint pid = _e2_command_run (args, tmp, range, show, wait, command,
		run_in_shell ? "sh>" : ">");
	if (free)
		g_strfreev (args);
	return pid;
}

gint _e2_command_run (gchar **args, gint num, E2_CommandRange range,
	gboolean show, gboolean wait, gchar *command, gchar *prefix)
{
	if (wait)
		return _e2_command_run_wait (args, num, range, show, command, prefix);
	block ();
	
	GError *error = NULL;
	gint pid = 0, pin = 0, pout = 0, perr = 0;
	g_spawn_async_with_pipes (curr_view->dir, args, NULL,
		G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
		NULL, NULL, &pid, &pin, &pout, &perr, &error);

	if (error != NULL)
	{
		gchar *message = g_strconcat (prefix, command, NULL);
		e2_output_print (message, NULL, FALSE, FALSE, TRUE, "blue", "bold", NULL);
		g_free (message);
		message = g_strconcat (error->message, ".\n", NULL);
		e2_output_print (message, NULL, TRUE, FALSE, FALSE, NULL);
		g_free (message);
		g_error_free (error);
		g_free (command);
		unblock ();
		return 0;
	}

	GIOChannel *ioc = g_io_channel_unix_new (pin);
	g_io_channel_set_encoding (ioc, NULL, NULL);
	g_io_channel_set_buffered (ioc, FALSE);
	g_io_channel_set_close_on_unref (ioc, TRUE);

	E2_CommandRuntime *rt = NULL;
	while ((rt = register_child (pid, 1, command, 0, ioc)) == NULL);
	rt->show = show;
	rt->closed = FALSE;

	if (rt->show)
	{
		gchar *message = g_strconcat (prefix, command, NULL);
		e2_output_print (message, rt->pidstr, FALSE, FALSE, FALSE, "blue", "bold", NULL);
		g_free (message);
		message = g_strdup_printf (" (%d)\n", pid);
		e2_output_print (message, rt->pidstr, FALSE, FALSE, FALSE, "small", "grey", NULL);
		g_free (message);
	}

	set_up_io_channel (pout, G_IO_IN | G_IO_PRI | G_IO_ERR |
		G_IO_HUP | G_IO_NVAL, watch_std, rt);
	//error messages will come through, no matter what
	set_up_io_channel (perr, G_IO_IN | G_IO_PRI | G_IO_ERR |
		G_IO_HUP | G_IO_NVAL, watch_err, rt);

	g_free (command);

	unblock ();
	
	return pid;
}

gint _e2_command_run_wait (gchar **args, gint num, E2_CommandRange range,
	gboolean show, gchar *command, gchar *prefix)
{
	block ();
	if (show)
	{
		gchar *message = g_strconcat (prefix, command, NULL);
		e2_output_print (message, NULL, FALSE, FALSE, TRUE, "blue", "bold", NULL);
		g_free (message);
		WAIT_FOR_EVENTS;
	}

	GError *error = NULL;
	gint exit = 0;
	gchar *sout = NULL, *serr = NULL;
	g_spawn_sync (curr_view->dir, args, NULL,
		G_SPAWN_SEARCH_PATH, NULL, NULL, &sout, &serr, &exit, &error);
	
	if (show)
	{
		if (sout != NULL)
			e2_output_print (sout, NULL, FALSE, FALSE, FALSE, NULL);
	}
	if (serr != NULL)
		e2_output_print (serr, NULL, FALSE, FALSE, FALSE, NULL);
	if (show)
	{
		gchar *message = g_strdup_printf (">%s returned '%d'",
			command, WEXITSTATUS (exit));
		e2_output_print (message, NULL, FALSE, FALSE, TRUE, "small", "grey", NULL);
		g_free (message);
	}

	g_free (command);
	unblock ();
	return exit;
}

void e2_command_init ()
{
	set_sigchild_handler ();
	
	e2_option_int_register ("command-watch-priority", "command:options",
		"watch priority", "the watch priority of commands influences how fast "
		"program output is read from io channels; however, a too high priority "
		"might decrease gui responsivness and break automatic scrolling. "
		"(note: negative values mean high priority, positiv values mean low "
		"priority)", NULL, 300, -200, 400);
	e2_option_bool_register ("command-use-aliases", "command:options", "use aliases",
		"this is a general switch to turn on/off alias handling for commands.",
		NULL, TRUE);

	e2_option_bool_register ("fileop-background", "command:file operations", "run in background",
		"runs all file operations syncronously in the background, not blocking"
		" the interface", NULL, FALSE);
	e2_option_bool_register ("fileop-show", "command:file operations", "show commands",
		"show the commands that are run on the output pane", NULL, TRUE);

	e2_action_register ("command.list children", E2_ACTION_TYPE_ITEM,
		e2_command_list_children, NULL, FALSE);

	e2_alias_init (&app.aliases, "command.aliases");
	e2_command_line_init ();
	e2_complete_init ();
}

