/* $Id: e2_option.h,v 1.26 2004/08/07 22:42:50 cairo Exp $
 * 
 * Copyright (C) 2003-2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef E2_OPTION_H
#  define E2_OPTION_H

#include "stdio.h"
#include "emelfm2.h"

#define DEFAULT_CONFIG_FILE "config"
#define LINE_BUFFER_SIZE 4096

typedef enum
{
	//simple types
	E2_OPTION_TYPE_BOOL = 1,
	E2_OPTION_TYPE_INT,
	E2_OPTION_TYPE_STR,
	E2_OPTION_TYPE_SEL,
	E2_OPTION_TYPE_FONT,
	E2_OPTION_TYPE_COLOR,
	E2_OPTION_TYPE_FILE,
	E2_OPTION_TYPE_ICON,
	E2_OPTION_TYPE_TREE,
} E2_OptionType;

typedef enum
{
	E2_OPTION_TREE_UP_DOWN = 1 << 0,
	E2_OPTION_TREE_ADD_DEL = 1 << 1,
	E2_OPTION_TREE_LAST_COL_EMPTY = 1 << 2,
	E2_OPTION_TREE_LIST = 1 << 3,
} E2_OptionTypeTreeFlags;

typedef struct _E2_OptionTypeExtraInt
{
	gint min;
	gint max;
} E2_OptionTypeExtraInt;

typedef struct _E2_OptionTypeExtraSel
{
	gchar **def;
} E2_OptionTypeExtraSel;

typedef struct _E2_OptionTypeExtraTree
{
	gpointer model;
	//holds default values for the tree
	gchar **def;
	//holds tree values in case the config file
	//has been read before the tree option was registed
	gchar **unknown;
	gint def_num;
	gboolean found_in_config_file;
	gint columns_num;
	GList *columns;
	gpointer selection_check_func;
	gpointer draggable_check_func;
	E2_OptionTypeTreeFlags flags;
} E2_OptionTypeExtraTree;

typedef struct _E2_OptionTypeExtraColor
{
	GdkColor value;
} E2_OptionTypeExtraColor;

typedef struct _E2_OptionSet
{
	E2_OptionType type;
	gchar *name;
	gchar *group;
	gchar *desc;
	gchar *tip;
	gchar *depends;
	
	gint idef;
	gchar *sdef;
	//FIXME: use ival and sval to save the actuall values of the option
	//so we can later use idef and sdef to get the defaults back for the option
	gint ival;
	gchar *sval;
	
	//extra option data
	union _E2_OptionTypeExtra
	{
		E2_OptionTypeExtraInt num;
		E2_OptionTypeExtraSel sel;
		E2_OptionTypeExtraTree tree;
		E2_OptionTypeExtraColor color;
	} ex;
	//last widget created from this option
	GtkWidget *widget;
	gboolean should_be_freed;

	gboolean hook_freezed;
	GHookList hook_value_changed;
} E2_OptionSet;

GHashTable *options_hash;
GHashTable *options_queue;
GPtrArray *options_array;

void e2_option_init ();
void e2_option_read_from_string (gchar *f[]);
gboolean e2_option_register (E2_OptionSet *set);
E2_OptionSet *e2_option_get (gchar *option);
E2_OptionSet *_e2_option_get (gchar *option);
void *_e2_option_void_get (E2_OptionSet *set, gchar *option);
void e2_option_clear ();
void e2_option_attach_value_changed (gchar *option, GtkWidget *widget,
	gpointer func, gpointer data);
void _e2_option_attach_value_changed (E2_OptionSet *set, GtkWidget *widget,
	gpointer func, gpointer data);
void destroy_config ();

#include "e2_option_bool.h"
#include "e2_option_int.h"
#include "e2_option_str.h"
#include "e2_option_sel.h"
#include "e2_option_color.h"
#include "e2_option_tree.h"
#include "e2_option_unknown.h"

  /***********************************************/
 /***** functions from e2_option__default.c *****/
/***********************************************/

void e2_option_default_register ();

  /******************************************/
 /***** functions from e2_option__io.c *****/
/******************************************/

void e2_option_read_from_file (gchar *fn);
void e2_option_write_to_file ();

  /**********************************************/
 /***** functions from e2_option__widget.c *****/
/**********************************************/

GtkWidget *e2_option_create_menu (GtkWidget *controller, GtkWidget *menu,
	E2_OptionSet *set, ...);
void e2_option_connect (GtkWidget *controller, gboolean  active);

#endif /* ifndef E2_OPTION_H */
