/* $Id: e2_option__default.c,v 1.20 2004/07/14 23:20:05 cairo Exp $
 * 
 * Copyright (C) 2003-2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/**
 * @file src/config/e2_option__default.c 
 * @brief default place to register miscellaneous options
 * 
 * default place to register miscellaneous options. (currently
 * most of the options are still registered here, but that is
 * going to change)
 *
 **/

#include <stdio.h>
#include "emelfm2.h"
#include "e2_option.h"
//#include "e2_action.h"

void e2_option_default_register ()
{
	E2_OptionSet *set;
	
	  /*******************/
	 /***** general *****/
	/*******************/
	e2_option_bool_register ("transparent-dir-links", "general:options", "transparent directory links",
		"when you change to a directory through a link and press updir later on, you'll return to"
		"the directory where the link was in instead of the real parent directory.", NULL, TRUE);
	e2_option_bool_register ("auto-refresh-config", "general:options", "reload config on external change",
		"automatically check if the config directory was changed by another program instance and"
		" reload it if necessary", NULL, TRUE);
	e2_option_bool_register ("relative-symlinks", "general:options", "relative symlinks",
		"when creating symlinks, make them relative", NULL, FALSE);
	gchar *opt_filelist_date_string[] = {" Default: May 20 09:11", "Standard: 20/05/04 09:11",
		"American: 05/20/04 09:11", NULL};
	e2_option_sel_register ("date-string", "general:options", "Date Format",
		"default format of the date in the panes", NULL, 0,
		opt_filelist_date_string);
	e2_option_bool_register ("use-external-viewer", "general:commands", "use external viewer",
		"if activated, the command entered below will be used for viewing files, "
		"otherwise the internal view dialog will be used", NULL, TRUE);
	e2_option_str_register ("command-viewer", "general:commands", "viewer command:",
		"sets the program to load as external viewer, the file will be the "
		"first argument", "use-external-viewer", "gview");
	e2_option_str_register ("command-xterm", "general:commands", "x terminal emulator:",
		"sets the program to load as terminal emulator", NULL, "xterm");
	e2_option_bool_register ("confirm-delete", "general:confirmation", "confirm delete",
		"pop up dialog before actually deleting anything", NULL, TRUE);
	e2_option_bool_register ("confirm-overwrite", "general:confirmation", "confirm overwrite",
		"pop up dialog before actually overwriting anyting", NULL, TRUE);
	
	  /*****************/
	 /***** panes *****/
	/*****************/
	e2_option_bool_register ("panes-horizontal", "panes:options", "use horizontal panes",
		"horizontal panes let you view more columns at once", NULL, FALSE);
	e2_option_bool_register ("auto-refresh", "panes:options", "auto refresh",
		"automatically check if directory contents has changed", NULL, TRUE);
	gchar *opt_scrollbar_position[] =
		{"bottom-right", "top-right", "bottom-left", "top-left", NULL};
	e2_option_sel_register ("scrollbar-position", "panes:options", "scrollbar position",
		"the default (bottom-right) should be ok for most users", NULL, 4, opt_scrollbar_position);
	e2_option_bool_register ("show-updir-entry", "panes:misc", "show updir entry .. in file list",
		"this is slow, you shouldn't use it", NULL, TRUE);
	e2_option_bool_register ("button2-updir", "panes:misc", "updir on button2 click",
		"this is a lot fast than double clicking '..' or pressing the updir button", NULL, FALSE);
	
	  /*********************/
	 /***** interface *****/
	/*********************/
	e2_option_bool_register ("use-vi-keys", "interface:misc", "use vi keybindings for file view",
		"type 'keys' on the command line after activating this to get an "
		"overview of the keys", NULL, FALSE);
	e2_option_bool_register ("windows-right-click", "interface:misc",
		"windows right click behaviour",  "if activated, clicking the right "
		"mouse button will also select the row under it", NULL, FALSE);
	e2_option_bool_register ("windows-right-click-extra", "interface:misc",
		"advanced windows right click behaviour",
		"clicks on free areas will not unselect the currently selected row",
		"windows-right-click", FALSE);
	e2_option_font_register ("list-font", "interface:fonts", "list font",
		"list font", NULL, "Sans 10");
	e2_option_font_register ("output-font", "interface:fonts", "output font",
		"output font", NULL, "Monospace 10");

	  /***************************/
	 /***** interface.menus *****/
	/***************************/
	e2_option_bool_register ("menu-show-icons", "interface.menus:style",
		"show icons in menus",  "usually icons are a nice thing to have because "
		"they help you recognize known items faster", NULL, TRUE);
	gchar *opt_menu_isize[] =
		{"menu", "toolbar small", "toolbar large", "button", "dnd", "dialog", NULL};
	e2_option_sel_register ("menu-isize", "interface.menus:style", "menu icon size",
		"this option will change the icon size for all menus, so be carefull! "
		"the default icon size for menu is 'menu' (how surprising)",
		"menu-show-icons", 0, opt_menu_isize);
	e2_option_int_register ("submenu-up-delay", "interface.menus:delays",
		"menu popup delay (ms)",
		"number of milliseconds to wait before poping up submenus",
		NULL, 50, 0, 10000);
	e2_option_int_register ("submenu-down-delay", "interface.menus:delays",
		"menu popdown delay (ms)", "number of milliseconds to wait before poping "
		"down activated submenus when the mouse pointer leaves the menu entry",
		NULL, 400, 0, 10000);
	
	  /****************************/
	 /***** interface.colors *****/
	/****************************/
	e2_option_color_register ("color-active-pane", "interface.colors 1:misc", "active pane color",
		"color of the column header of the active pane", NULL, "dark grey");
	e2_option_color_register ("color-drag-highlight", "interface.colors 1:misc", "drag highlight color",
		"color to mark possible destination row of a drag 'n' drop operation", NULL, "wheat");
	e2_option_color_register ("color-inactive-selected", "interface.colors 1:misc", "inactive selected color",
		"color to mark selected files and directories in the inactive pane", NULL, "light grey");
	e2_option_color_register ("color-tag", "interface.colors 1:misc", "tagged row color",
		"color of tagged rows", NULL, "yellow");
	e2_option_color_register ("color-ft-exec", "interface.colors 2:file type", "executable color",
		"color of executable files", NULL, "forest green");
	e2_option_color_register ("color-ft-dir", "interface.colors 2:file type", "directory color",
		"color of directories", NULL, "blue");
	e2_option_color_register ("color-ft-link", "interface.colors 2:file type", "symlink color",
		"color of symbolic links", NULL, "sky blue");
	e2_option_color_register ("color-ft-dev", "interface.colors 2:file type", "device color",
		"color of devices", NULL, "orange");
	e2_option_color_register ("color-ft-socket", "interface.colors 2:file type", "socket color",
		"color of sockets", NULL, "purple");
	e2_option_color_register ("color-positive", "interface.colors 3:misc",
		_("positive color"), _("This is the color to mark messages of "
		"successful operations or other positive events."), NULL, "dark green");
	e2_option_color_register ("color-negative", "interface.colors 3:misc",
		_("negative color"), _("This is the color to mark messages of unsuccessful "
		"operations or other negative events."), NULL, "red");
	e2_option_color_register ("color-unimportant", "interface.colors 3:misc",
		_("unimportant color"), _("This is the color to mark messages of minor "
		"importance or other miscellaneous events."), NULL, "light grey");
	
	  /*******************/
	 /***** toolbar *****/
	/*******************/
	set = e2_option_tree_register ("toolbar", "toolbar", "toolbar", NULL,
		NULL, NULL, E2_OPTION_TREE_UP_DOWN | E2_OPTION_TREE_ADD_DEL);
	e2_option_tree_add_column (set, "name", E2_OPTION_TREE_TYPE_STR, 0,
		"separator", NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, "icon", E2_OPTION_TREE_TYPE_ICON, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, "tooltip", E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, "action", E2_OPTION_TREE_TYPE_SEL,
		0, "<separator>", e2_actions_fill_store,
		(void *) E2_ACTION_EXCLUDE_TOOLBAR, 0, NULL, NULL);
	e2_option_tree_add_column (set, "argument", E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_sync_columns (set);
	_e2_option_tree_append_default (set,
		"<toolbar",
		"bookmarks|||bookmark.<widget>|toolbar folder",
		"separator|||<separator>|",
		"refresh|gtk-refresh||panes.refresh|",
		"copy|gtk-copy||file.copy|",
		"move|gtk-cut||file.move|",
		"\\<span foreground='red'><b>delete</b></span>|gtk-delete||file.delete|",
		"rename|gtk-index||file.rename|",
		"mkdir|gtk-open|open mkdir dialog|file.mkdir|",
		"separator|||<separator>|",
		"config|gtk-preferences|open config dialog|config.configure|",
		"separator|||<separator>|",
		"quit|gtk-quit|quit emelFM2|quit|",
		">", NULL);
	
	  /**********************/
	 /***** commandbar *****/
	/**********************/
	set = e2_option_tree_register ("commandbar", "commandbar", "commandbar", NULL,
		NULL, NULL, E2_OPTION_TREE_UP_DOWN | E2_OPTION_TREE_ADD_DEL);
	e2_option_tree_add_column (set, "name", E2_OPTION_TREE_TYPE_STR, 0,
		"separator", NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, "icon", E2_OPTION_TREE_TYPE_ICON, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, "tooltip", E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, "action", E2_OPTION_TREE_TYPE_SEL,
		0, "<separator>", e2_actions_fill_store, (void *)
		E2_ACTION_EXCLUDE_TOOLBAR, 0, NULL, NULL);
	e2_option_tree_add_column (set, "argument", E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_sync_columns (set);
	_e2_option_tree_append_default (set,
		"<commandbar",
		"max|gtk-goto-top|un-/maximize output pane|output.adjust ratio|*,0",
		"$(USER)@$(HOSTNAME):|gtk-go-up|show/hide output pane|output.adjust ratio|*,100%",
		"command line|||command line.<widget>|100,expand",
		"|gtk-goto-first|clear command line|command line.clear|",
		"clear|gtk-clear|clear output pane|output.clear|",
		"ps||list child processes|command.list children|",
		"du||disk usage plugin|Disk Usage|",
		"find|gtk-find|open find dialog|find|",
		"x||xterm|<custom command>|$[command-xterm] &",
		"su||run something as root|<custom command>|su %{enter command:}; echo \"done. press enter\"; read",
		"|gtk-dialog-info|show about dialog|dialog.about|",
		"|gtk-quit|quit emelFM2|quit|",
		">", NULL);
	
	  /************************/
	 /***** context menu *****/
	/************************/
	set = e2_option_tree_register ("context-menu", "context menu", "context menu",
		NULL, NULL, NULL, E2_OPTION_TREE_UP_DOWN | E2_OPTION_TREE_ADD_DEL);
	e2_option_tree_add_column (set, "name", E2_OPTION_TREE_TYPE_STR, 0,
		"separator", NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, "icon", E2_OPTION_TREE_TYPE_ICON, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, "shift", E2_OPTION_TREE_TYPE_BOOL, FALSE,
		"false", NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, "ctrl", E2_OPTION_TREE_TYPE_BOOL, FALSE,
		"false", NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, "action", E2_OPTION_TREE_TYPE_SEL,
		0, "<separator>", e2_actions_fill_store, (void *)
		E2_ACTION_EXCLUDE_MENU, 0, NULL, NULL);
	e2_option_tree_add_column (set, "argument", E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_sync_columns (set);
	_e2_option_tree_append_default (set,
		"<context-menu",
		"file actions_||false|false|<file actions>|",
		"separator||false|false|<separator>|",
		"edit filetype|gtk-properties|false|false|config.filetype|",
		"open _with||false|false|open with|",
		"_view||false|false|view|",
		"separator||false|false|<separator>|",
		"_file actions|gtk-add|false|false|<submenu>|",
		"	file _info|gtk-dialog-info|false|false|file.file info|",
		"	_permissions||false|false|file.permission|",
		"	_user-group||false|false|file.user/group|",
		"	_rename||false|false|file.rename|",
		"	_copy|gtk-copy|false|false|file.copy|",
		"	_move||false|false|file.move|",
		"	_delete|gtk-delete|false|false|file.delete|",
		"	move as||false|false|file.move as|",
		"	s_ymlink||false|false|file.symlink|",
		"	symlink as||false|false|file.symlink as|",
		"_mkdir|gtk-open|false|false|file.mkdir|",
		"separator||false|false|<separator>|",
		"bookmarks|gtk-jump-to|false|false|<submenu>|",
		"	add bookmark||false|false|bookmark.add|",
		"	add to top||false|false|bookmark.add|top",
		"	edit bookmarks||false|false|config.configure|bookmarks",
		"	separator||false|false|<separator>|",
		"	bookmarks||false|false|bookmark.<widget>|",
		"user commands||false|false|<submenu>|",
		"	new file|gtk-new|false|true|<custom command>|touch %{enter file name:}",
		"	easytag||false|true|<custom command>|easytag %d &",
		"	remove spaces||false|true|<custom command>|mv %f `echo %f \\|sed -e 's/ //g'` 2>/dev/null &",
		"	separator||false|false|<separator>|",
		"	show toolbar||false|false|toggle toolbar|",
		"	show command bar||false|false|toggle commandbar|",
		"plugins|gtk-convert|false|false|<submenu>|",
		"	edit plugins|gtk-preferences|true|false|config.configure|Plugins",
		"	separator||true|false|<separator>|",
		"	plugins||true|false|<plugins>|",
		"separator||true|false|<separator>|",
		"configure|gtk-preferences|false|false|config.configure|",
		">", NULL);
}
