/*
 * $Id: e2_option_color.c,v 1.3 2004/05/23 02:10:18 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of config.
 * config is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * config is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "emelfm2.h"
#include "e2_option.h"
#include "e2_option_color.h"
#include "debug.h"

E2_OptionSet *e2_option_color_register (gchar *name, gchar *group, gchar *desc,
	gchar *tip, gchar *depends, gchar *def)
{
	E2_OptionSet *set;
	if ((set = _e2_option_get (name)) != NULL)
		return set;
	set = malloc (sizeof (E2_OptionSet));
	//set up E2_OptionSet
	set->name = name;
	set->group = group;
	set->type = E2_OPTION_TYPE_COLOR;
	set->desc = desc;
	set->tip = tip;
	set->depends = depends;
	set->sdef = g_strdup (def);
	if (!gdk_color_parse (set->sdef, &set->ex.color.value))
	{
		g_free (set->sdef);
		set->sdef = g_strdup ("#000000");
		gdk_color_parse (set->sdef, &set->ex.color.value);
	}
	e2_option_register (set);
	return set;
}

GdkColor *e2_option_color_get (gchar *name)
{
	E2_OptionSet *set = e2_option_get (name);
	if (set == NULL)
		return NULL;
	if (set->type == E2_OPTION_TYPE_COLOR)
	{
		return &set->ex.color.value;
	} else
	{
		printd (WARN, "trying to get color option '%s', which isn't a color option", set->name);
		return NULL;
	}
}

gboolean e2_option_color_set_str (gchar *name, gchar *value)
{
	E2_OptionSet *set = e2_option_get (name);
	if (set->type == E2_OPTION_TYPE_COLOR)
	{
		return _e2_option_color_set_str (set, value);
	} else
	{
		printd (WARN, "trying to set color option '%s', which isn't a color option", set->name);
		return FALSE;
	}
}

gboolean _e2_option_color_set_str (E2_OptionSet *set, gchar *value)
{
	if (!gdk_color_parse (value, &set->ex.color.value))
	{
		gdk_color_parse (set->sdef, &set->ex.color.value);
		return FALSE;
	}
	g_free (set->sdef);
	set->sdef = g_strdup (value);
	return TRUE;
}
