/*
 * $Id: e2_option_int.c,v 1.7 2004/05/23 02:10:18 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of config.
 * config is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * config is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include "emelfm2.h"
#include "e2_option.h"
#include "e2_option_int.h"
#include "debug.h"

void e2_option_int_add_widget (GtkWidget *dialog, GtkWidget *box,
	GtkSizeGroup *size_group, E2_OptionSet *set)
{
	GtkWidget *event = e2_widget_add_eventbox (box, TRUE, 0);
	GtkWidget *hbox = gtk_hbox_new (FALSE, 5);
	gtk_container_add (GTK_CONTAINER (event), hbox);
	gchar *label_text = g_strconcat (set->desc, ":", NULL);
	GtkWidget *label = e2_widget_add_label (hbox, label_text, 0.0, FALSE, 0);
	g_free (label_text);
	gtk_size_group_add_widget (size_group, label);
	gdouble max_percent = set->ex.num.max / 100.0;
	GtkAdjustment *spinner_adj = (GtkAdjustment *) gtk_adjustment_new
		(set->idef, set->ex.num.min, set->ex.num.max, 1.0, max_percent, max_percent);
	set->widget = gtk_spin_button_new (spinner_adj, 1.0, 0);
	e2_widget_set_tooltip (NULL, event, set->tip);
	gtk_box_pack_start (GTK_BOX (hbox), set->widget, FALSE, FALSE, 5);

}

E2_OptionSet *e2_option_int_register (gchar *name, gchar *group, gchar *desc,
	gchar *tip, gchar *depends, gint def, gint min, gint max)
{
	E2_OptionSet *set;
	if ((set = _e2_option_get (name)) != NULL)
		return set;
	set = malloc (sizeof (E2_OptionSet));
	//set up E2_OptionSet
	set->name = name;
	set->group = group;
	set->type = E2_OPTION_TYPE_INT;
	set->desc = desc;
	set->tip = tip;
	set->depends = depends;
	set->idef = 0;
	set->sdef = NULL;
	set->ex.num.min = min;
	set->ex.num.max = max;
	_e2_option_int_set (set, def);
	e2_option_register (set);
	return set;
}

gint e2_option_int_get (gchar *option)
{
	E2_OptionSet *set;
	set = e2_option_get (option);
	if ((set->type == E2_OPTION_TYPE_SEL) || (set->type == E2_OPTION_TYPE_INT))
	{
		return _e2_option_int_get (set);
	} else
	{
		printd (WARN, "trying to get int option '%s' which isn't an int",
			option);
		return -1;
	}
}

gint _e2_option_int_get (E2_OptionSet *set)
{
	return set->idef;
}

gint e2_option_int_set (gchar *option, gint value)
{
	E2_OptionSet *set;
	set = e2_option_get (option);
	if (set->type == E2_OPTION_TYPE_INT)
		return _e2_option_int_set (set, value);
	else
	{
		printd (WARN, "trying to set int option '%s' which isn't an int",
			option);
		return 0;
	}
}

gint _e2_option_int_set (E2_OptionSet *set, gint value)
{
	if (value < set->ex.num.min) value = set->ex.num.min;
	if (value > set->ex.num.max) value = set->ex.num.max;
	set->idef = value;
	g_free (set->sdef);
	set->sdef = (gchar *) g_strdup_printf ("%d", set->idef);
	return value;
}
