/*
 * $Id: e2_option_str.c,v 1.5 2004/05/23 02:10:18 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of config.
 * config is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * config is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "emelfm2.h"
#include "e2_option.h"
#include "e2_option_str.h"

E2_OptionSet *e2_option_str_register (gchar *name, gchar *group, gchar *desc,
	gchar *tip, gchar *depends, gchar *def)
{
	E2_OptionSet *set;
	if ((set = _e2_option_get (name)) != NULL)
		return set;
	set = malloc (sizeof (E2_OptionSet));
	//set up E2_OptionSet
	set->name = name;
	set->group = group;
	set->type = E2_OPTION_TYPE_STR;
	set->desc = desc;
	set->tip = tip;
	set->depends = depends;
	set->sdef = g_strdup (def);
	e2_option_register (set);
	return set;
}

E2_OptionSet *e2_option_font_register (gchar *name, gchar *group, gchar *desc,
	gchar *tip, gchar *depends, gchar *def)
{
	E2_OptionSet *set = e2_option_str_register (name, group, desc, tip,
		depends, def);
	set->type = E2_OPTION_TYPE_FONT;
	return set;
}

gchar *e2_option_str_get (gchar *option)
{
	E2_OptionSet *set = e2_option_get (option);
	return set->sdef;
}

gchar *_e2_option_str_get (E2_OptionSet *set)
{
	return set->sdef;
}

void _e2_option_str_set (E2_OptionSet *set, gchar *value)
{
	g_free (set->sdef);
	set->sdef = g_strdup (value);
}

//return true if option was set successfully
gboolean e2_option_set_str (gchar *option, gchar *str)
{
	E2_OptionSet *set = _e2_option_get (option);
	if (set == NULL)
	{
		e2_option_unknown_register (option, str);
		return TRUE;
	}
	return _e2_option_set_str (set, str);
}

gboolean _e2_option_set_str (E2_OptionSet *set, gchar *str)
{
	switch (set->type)
	{
		case E2_OPTION_TYPE_BOOL:
			_e2_option_bool_set (set, g_str_equal (str, "true") ? TRUE : FALSE);
			break;
		case E2_OPTION_TYPE_INT:
		{
			//FIXME: check for conversion success
			gint i = (gint) g_ascii_strtoull (str, NULL, 10);
			if ((i > set->ex.num.max) || (i < set->ex.num.min)) return FALSE;
			set->idef = i;
			set->sdef = g_strdup (str);
			break;
		}
		case E2_OPTION_TYPE_STR:
		case E2_OPTION_TYPE_FONT:
			_e2_option_str_set (set, str);
			break;
		case E2_OPTION_TYPE_COLOR:
			return _e2_option_color_set_str (set, str);
			break;
		case E2_OPTION_TYPE_SEL:
		{
			gchar *val;
			gint i = 0;
			while ((val = set->ex.sel.def[i]) != NULL)
			{
				if (g_str_equal (str, val))
				{
					set->idef = i;
					set->sdef = val;
					break;
				}
				i++;
			}
		}
			break;
		default:
			break;
	}
	return TRUE;
}

