/*
 * $Id: e2_option_unknown.c,v 1.3 2004/05/23 02:10:18 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of config.
 * config is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * config is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include "emelfm2.h"
#include "e2_option.h"
#include "e2_option_unknown.h"

/**
 * @brief register an unkown option
 *
 * register an yet unknown option to use the data later on. this function
 * is supposed to be used internally only. already existing unknown options
 * will be removed first.
 *
 * @param option  string with the name of the option
 * @param value  option value; will be duplicated
 *
 * @return 
 **/
void e2_option_unknown_register (gchar *option, gchar *value)
{
	gchar *value_old;
	if ((value_old = g_hash_table_lookup (options_queue, option)) == NULL)
	{
		g_free (value_old);
		g_hash_table_remove (options_queue, option);
	}
	g_hash_table_insert (options_queue, g_strdup (option), g_strdup (value));
}

/**
 * @brief register an unknown option
 *
 * register an yet unknown option that is several lines long. this function
 * is currently only used for tree options. this function is supposed to be
 * used internally only. already existing unknown options will be removed
 * first.
 * 
 * @param option  string with the name of the option
 * @param value  array of strings in config file format
 *
 * @return 
 **/
void e2_option_unknown_register2 (gchar *option, gchar **value)
{
	gchar **value_old;
	if ((value_old = g_hash_table_lookup (options_queue, option)) == NULL)
	{
		g_strfreev (value_old);
		g_hash_table_remove (options_queue, option);
	}
	g_hash_table_insert (options_queue, g_strdup (option), value);
}
