/* $Id: e2_config_dialog.c,v 1.14 2004/07/28 20:29:01 cairo Exp $
 *
 * Copyright (C) 2003-2004 tooar <tooar@gmx.net>
 * Copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gprintf.h>
#include "emelfm2.h"
#include "build.h"
#include "e2_option.h"
#include "e2_action.h"
#include "e2_utils.h"
#include "e2_dialog.h"
#include "e2_button.h"
#include "e2_window.h"
#include "e2_output.h"
#include "e2_config_dialog.h"
#include "mg-cell-renderer-list.h"

//dialogs
static GtkWidget *config_dialog = NULL;

static GList *added_plugins, *removed_plugins;

//static GtkWidget *bookmarks_clist;
static GtkWidget *filetypes_clist;
//static GtkWidget *keys_clist;
static GtkWidget *plugins_clist;
static GtkWidget *file_selection;
static GtkWidget *column_checks[MAX_COLUMNS];

//common vars we need in several functions
static GHashTable *fast;
static GtkWidget *treeview;
static GtkTreeIter page_last_iter;
static gint page_last;
static gchar *page_selected;

  /*********************/
 /***** callbacks *****/
/*********************/

static void show_cb (GtkWidget *dialog, GtkWidget *sw)
{
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
}

//callback on changes on the dependencies between option widgets
static gboolean depends_cb (GtkWidget *button, GtkWidget *data)
{
	gtk_widget_set_sensitive (data,
		!GTK_TOGGLE_BUTTON (button)->active);
	return TRUE;
}
static gboolean depends_cb_ (GtkWidget *button, GtkWidget *data)
{
	gtk_widget_set_sensitive (data,
		GTK_TOGGLE_BUTTON (button)->active);
	return TRUE;
}

static void e2_cd_clean ()
{
	if (config_dialog != NULL)
	{
		gtk_widget_destroy (config_dialog);
		config_dialog = NULL;
	}
}

gboolean e2_cd_ok_cb (GtkWidget *button, gboolean apply)
{
	printd (DEBUG, "callback: ok");
	gint i;
	GList *tmp;

	if (!apply)
	{
		e2_list_free_with_data (&cfg.filetypes);
//		e2_list_free_with_data (&cfg.key_bindings);
		e2_list_free_with_data (&cfg.plugins); 
		cfg.filetypes = clist_data_to_glist (filetypes_clist);
//		cfg.key_bindings = clist_data_to_glist (keys_clist);
		cfg.plugins = clist_data_to_glist (plugins_clist);

		for (tmp = removed_plugins; tmp != NULL; tmp = tmp->next)
			destroy_plugin ((Plugin*)tmp->data);
		g_list_free (removed_plugins);
		g_list_free (added_plugins);
	}
	
	for (i = 0; i < MAX_COLUMNS; i++)
		all_columns[i].is_visible = GTK_TOGGLE_BUTTON (column_checks[i])->active;

	//update options
	for (i = 0; i < options_array->len; i++)
	{
		E2_OptionSet *set = g_ptr_array_index (options_array, i);
		switch (set->type)
		{
			case E2_OPTION_TYPE_BOOL:
				e2_option_bool_set (set->name, 
					gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (set->widget)));
				break;
			case E2_OPTION_TYPE_INT:
				_e2_option_int_set (set, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (set->widget)));
				break;
			case E2_OPTION_TYPE_FONT:
			case E2_OPTION_TYPE_COLOR:
			case E2_OPTION_TYPE_STR:
				{
					gchar *chars = gtk_editable_get_chars (GTK_EDITABLE (set->widget), 0, -1);
					gchar *utf = UTF2STR (chars);
					e2_option_set_str (set->name, utf);
					g_free (utf);
					g_free (chars);
				}
				break;
			case E2_OPTION_TYPE_SEL:
				_e2_option_sel_set (set, gtk_combo_box_get_active (GTK_COMBO_BOX (set->widget)));
				break;
			default:
				break;
		}
	}

		if (cfg.check_config_id != 0)
			gtk_timeout_remove (cfg.check_config_id);
		if (e2_option_bool_get ("auto-refresh-config"))
			cfg.check_config_id = gtk_timeout_add (1000, check_config_files,
				NULL);
		if (cfg.auto_refresh_id != 0)
			gtk_timeout_remove (cfg.auto_refresh_id);
		if (e2_option_bool_get ("auto-refresh"))
			cfg.auto_refresh_id = gtk_timeout_add (1000, auto_refresh, NULL);

	if (!apply)
		e2_cd_clean ();

	e2_window_recreate (&app.window);

	//save
	if (!apply)
	{
	write_filetypes_file ();
//	write_bookmarks_file ();
//	write_keys_file ();
	write_plugins_file ();
	write_config_file ();
	e2_option_write_to_file ();
	touch_config_dir ();
	}
	return TRUE;
}

gboolean e2_cd_cancel_cb (GtkWidget *button)
{
	if (config_dialog != NULL)
	{
		GList *tmp;

		e2_list_free_with_data (&removed_plugins);
		for (tmp = added_plugins; tmp != NULL; tmp = tmp->next)
			destroy_plugin ((Plugin*)tmp->data);

		g_list_free (added_plugins);

		free_clist_data (filetypes_clist);
//		free_clist_data (keys_clist);

		e2_cd_clean ();
		gtk_widget_grab_focus (curr_view->clist);
	}
	return TRUE;
}

static void response_cb (GtkWidget *button, gint response,
	E2_ConfigDialogRuntime *rt)
{
	switch (response)
	{
		case GTK_RESPONSE_OK:
			e2_cd_ok_cb (button, FALSE);
			g_free (rt);
			break;
		case GTK_RESPONSE_APPLY:
			e2_cd_ok_cb (button, TRUE);
			break;
		case GTK_RESPONSE_CANCEL:
			e2_cd_cancel_cb (button);
			g_free (rt);
			break;
		default:
			break;
	}
	return;
}

static gboolean key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer data)
{
	if (event->keyval == GDK_Return)
		return e2_cd_ok_cb (NULL, FALSE);
	if (event->keyval == GDK_Escape)
		return e2_cd_cancel_cb (NULL);
	return FALSE;
}

// FileType Page Callbacks
static gboolean add_filetype_cb (GtkWidget *button, GtkWidget *clist)
{
	FileType *ft = NULL;

	gtk_widget_set_sensitive (config_dialog, FALSE);
	create_filetype_dialog (&ft, TRUE);
	gtk_main ();

	if (ft != NULL)
	{
		gchar *buf[2];
		gint i;

		buf[0] = ft->extensions;
		buf[1] = ft->description;
		i = gtk_clist_append (GTK_CLIST (clist), buf);
		gtk_clist_set_row_data (GTK_CLIST (clist), i, ft);
	}
	gtk_widget_set_sensitive (config_dialog, TRUE);
	return TRUE;
}

static gboolean edit_filetype_cb (GtkWidget *button, GtkWidget *clist)
{
	gint selected_row;
	FileType *ft;
	
	if (GTK_CLIST (clist)->selection != NULL)
		selected_row = (gint)GTK_CLIST (clist)->selection->data;
	else
		return TRUE;

	gtk_widget_set_sensitive (config_dialog, FALSE);
	ft = gtk_clist_get_row_data (GTK_CLIST (clist), selected_row);
	create_filetype_dialog (&ft, FALSE);
	gtk_main ();

	gtk_clist_set_text (GTK_CLIST (clist), selected_row, 0, ft->extensions);
	gtk_clist_set_text (GTK_CLIST (clist), selected_row, 1, ft->description);
	gtk_widget_set_sensitive (config_dialog, TRUE);
	return TRUE;
}

  /****************************/
 /***** plugin callbacks *****/
/****************************/

static gboolean add_plugin_ok_cb (GtkWidget *widget, GtkWidget *clist)
{
	printd (DEBUG, "callback: add plugin ok");
	Plugin *p;
	gchar *buf[2];
	gint row;

	p = load_plugin ((gchar *) gtk_file_selection_get_filename
		(GTK_FILE_SELECTION (file_selection)));
	if (p == NULL)
	{
		status_message ("Unable to load module.\n");
		gtk_widget_set_sensitive (config_dialog, TRUE);
		gtk_widget_destroy (file_selection);
		return TRUE;
	}

	buf[0] = p->name;
	buf[1] = _("YES");
	p->show_in_menu = TRUE;
	row = gtk_clist_append (GTK_CLIST (clist), buf);
	gtk_clist_set_row_data (GTK_CLIST (clist), row, p);
	added_plugins = g_list_append (added_plugins, p);

	gtk_widget_set_sensitive (config_dialog, TRUE);
	gtk_widget_destroy (file_selection);
	return TRUE;
}

static gboolean add_plugin_cancel_cb (GtkWidget *widget)
{
	gtk_widget_set_sensitive (config_dialog, TRUE);
	gtk_widget_destroy (file_selection);
	return TRUE;
}

static gboolean add_plugin_cb (GtkWidget *widget, GtkWidget *clist)
{
	gchar path[PATH_MAX];
	file_selection = gtk_file_selection_new ("select plugin to add");
	g_snprintf (path, sizeof (path), "%s/", PLUGINS_DIR);
	gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_selection), path);

	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selection)->ok_button),
		"clicked", G_CALLBACK (add_plugin_ok_cb), clist);
	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selection)->cancel_button),
		"clicked", G_CALLBACK (add_plugin_cancel_cb), clist);
	g_signal_connect (GTK_OBJECT (file_selection),
		"delete_event", G_CALLBACK (add_plugin_cancel_cb), clist);
	
	gtk_widget_show (file_selection);
	
	//setup dialog window
	gtk_window_set_modal (GTK_WINDOW (file_selection), TRUE);
	gtk_widget_set_sensitive (config_dialog, FALSE);
	gtk_window_set_transient_for (GTK_WINDOW (file_selection), GTK_WINDOW (config_dialog));
	
	return TRUE;
}

static gboolean plugin_info_cb (GtkWidget *widget, GtkWidget *clist)
{
	Plugin *p;
	gint selected_row;

	if (GTK_CLIST (clist)->selection != NULL)
		selected_row = (gint)GTK_CLIST (clist)->selection->data;
	else
		return TRUE;

	gtk_widget_set_sensitive (config_dialog, FALSE);
	p = gtk_clist_get_row_data (GTK_CLIST (clist), selected_row);
	create_plugin_info_dialog (&p);
	gtk_main ();
	gtk_clist_set_text (GTK_CLIST (clist), selected_row, 1,
				            (p->show_in_menu ? "YES" : "NO"));
	gtk_widget_set_sensitive (config_dialog, TRUE);
	return TRUE;
}

static gboolean remove_plugin_cb (GtkWidget *widget, GtkWidget *clist)
{
	Plugin *p;
	gint selected_row;

	if (GTK_CLIST (clist)->selection != NULL)
		selected_row = (gint)GTK_CLIST (clist)->selection->data;
	else
		return TRUE;

	p = gtk_clist_get_row_data (GTK_CLIST (clist), selected_row);
	removed_plugins = g_list_append (removed_plugins, p);
	gtk_clist_remove (GTK_CLIST (clist), selected_row);
	return TRUE;
}

  /**************************/
 /***** font callbacks *****/
/**************************/

static void font_select_ok_cb (GtkWidget *button, GtkWidget *entry)
{
	//FIXME: very bad
	GtkWidget *dialog = button->parent->parent->parent;
	const gchar *title = gtk_window_get_title (&(GTK_DIALOG (dialog)->window));
	gchar *font_str = gtk_font_selection_dialog_get_font_name
		(GTK_FONT_SELECTION_DIALOG (dialog));
	gtk_entry_set_text (GTK_ENTRY (entry), font_str);
	g_free (font_str);
	g_object_set_data (G_OBJECT (config_dialog), title, NULL);
}

static void font_select_cancel_cb (GtkWidget *button, GtkWidget *dialog)
{
	const gchar *title = gtk_window_get_title (&(GTK_DIALOG (dialog)->window));
	g_object_set_data (G_OBJECT (config_dialog), title, NULL);
}

static void font_select_cb (GtkWidget *button, E2_OptionSet *set)
{
	gchar *title = g_strconcat ("choose font: ", set->desc, NULL);
	GtkFontSelectionDialog *dialog = g_object_get_data (G_OBJECT (config_dialog), title);
	if (dialog != NULL)
		gtk_window_present (GTK_WINDOW (dialog));
	else
	{
		dialog = GTK_FONT_SELECTION_DIALOG (gtk_font_selection_dialog_new (title));
		gtk_widget_destroy (dialog->apply_button);
		gtk_font_selection_dialog_set_font_name (dialog,
			gtk_entry_get_text (GTK_ENTRY (set->widget)));
		g_signal_connect (G_OBJECT (dialog->ok_button), "clicked",
			G_CALLBACK (font_select_ok_cb), set->widget);
		g_signal_connect (G_OBJECT (dialog->cancel_button), "clicked",
			G_CALLBACK (font_select_cancel_cb), dialog);
		g_signal_connect (G_OBJECT (dialog), "delete_event",
			G_CALLBACK (font_select_cancel_cb), NULL);
		g_object_set_data_full (G_OBJECT (config_dialog), title, dialog, (void *) gtk_widget_destroy);
		e2_dialog_show (GTK_WIDGET (dialog), config_dialog, 0, 0);
	}
	g_free (title);
}

static void font_entry_cb (GtkEditable *entry, GtkWidget *label)
{
	gchar *font_str = gtk_editable_get_chars (entry, 0, -1);
	if ((font_str != NULL) && (*font_str != '\0') && (strlen (font_str) > 2))
	{
		gchar *label_text = g_strconcat ("\t<small><i>example:</i></small> <span font_desc=\"",
			font_str, "\">abcd efgh ABCD EFGH</span>\n", NULL);
		gtk_label_set_markup (GTK_LABEL (label), label_text);
		g_free (label_text);
	}
	g_free (font_str);
}

  /***************************/
 /***** color callbacks *****/
/***************************/

static void color_select_ok_cb (GtkWidget *button, GtkWidget *entry)
{
	GtkWidget *dialog = button->parent->parent->parent;
	const gchar *title = gtk_window_get_title (&(GTK_DIALOG (dialog)->window));
	GdkColor color;
	gtk_color_selection_get_current_color (GTK_COLOR_SELECTION 
		(GTK_COLOR_SELECTION_DIALOG (dialog)->colorsel), &color);
	gchar *color_str = e2_utils_color2str (&color);
	gtk_entry_set_text (GTK_ENTRY (entry), color_str);
	g_free (color_str);
	g_object_set_data (G_OBJECT (config_dialog), title, NULL);
}

static void color_select_cancel_cb (GtkWidget *button, GtkWidget *dialog)
{
	const gchar *title = gtk_window_get_title (&(GTK_DIALOG (dialog)->window));
	g_object_set_data (G_OBJECT (config_dialog), title, NULL);
}

static void color_select_cb (GtkWidget *button, E2_OptionSet *set)
{
	gchar *title = g_strconcat ("choose color: ", set->desc, NULL);
	GtkColorSelectionDialog *dialog = g_object_get_data (G_OBJECT (config_dialog), title);
	if (dialog != NULL)
		gtk_window_present (GTK_WINDOW (dialog));
	else
	{
		dialog = GTK_COLOR_SELECTION_DIALOG (gtk_color_selection_dialog_new (title));
		gtk_widget_destroy (GTK_COLOR_SELECTION_DIALOG (dialog)->help_button);

		GdkColor color;
		gchar *color_str = gtk_editable_get_chars (GTK_EDITABLE (set->widget), 0, -1);
		gdk_color_parse (color_str, &color);
		gtk_color_selection_set_current_color (GTK_COLOR_SELECTION (dialog->colorsel), &color);
		g_free (color_str);
		g_signal_connect (G_OBJECT (dialog->ok_button), "clicked",
			G_CALLBACK (color_select_ok_cb), set->widget);
		g_signal_connect (G_OBJECT (dialog->cancel_button), "clicked",
			G_CALLBACK (color_select_cancel_cb), dialog);
		g_signal_connect (G_OBJECT (dialog),
			"delete_event", G_CALLBACK (color_select_cancel_cb), dialog);
		g_object_set_data_full (G_OBJECT (config_dialog), title, dialog,
			(void *) gtk_widget_destroy);
		gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
		e2_dialog_show (GTK_WIDGET (dialog), config_dialog, 0, 0);
	}
	g_free (title);
}

static void color_entry_cb (GtkEditable *entry, GtkWidget *label)
{
	gchar *color_str = gtk_editable_get_chars (entry, 0, -1);
	GdkColor color;
	if ((gdk_color_parse (color_str, &color)) == TRUE)
	{
		gchar *label_text = g_strconcat ("\t"
			"<small><i>example:</i></small> " 
			"<span foreground=\"", color_str, "\" background=\"",
			color_str, "\">abCD</span> <span foreground=\"",
			color_str, "\">abCD</span><small>  |  ",
			"<i>current color:</i></small> ", color_str, 
			"\n", NULL);
		gtk_label_set_markup (GTK_LABEL (label), label_text);
		g_free (label_text);
	}
	g_free (color_str);
}

//clist callbacks
static void
clist_up_cb (GtkWidget *widget, GtkWidget *clist)
{
	gint selected_row;

	if (GTK_CLIST (clist)->selection != NULL)
		selected_row = (gint)GTK_CLIST (clist)->selection->data;
	else
		return;

	if (selected_row < 1)
		return;

	gtk_clist_swap_rows (GTK_CLIST (clist), selected_row,
				                                selected_row-1);
	gtk_clist_select_row (GTK_CLIST (clist), selected_row-1, 0);
}

static void
clist_down_cb (GtkWidget *widget, GtkWidget *clist)
{
	gint selected_row;

	if (GTK_CLIST (clist)->selection != NULL)
		selected_row = (gint)GTK_CLIST (clist)->selection->data;
	else
		return;

	if (selected_row >= (GTK_CLIST (clist)->rows - 1))
		return;

	gtk_clist_swap_rows (GTK_CLIST (clist), selected_row, selected_row+1);
	gtk_clist_select_row (GTK_CLIST (clist), selected_row+1, 0);
}

static void
clist_remove_cb (GtkWidget *widget, GtkWidget *clist)
{
	gint selected_row;

	if (GTK_CLIST (clist)->selection != NULL)
		selected_row = (gint)GTK_CLIST (clist)->selection->data;
	else
		return;

	g_free (gtk_clist_get_row_data (GTK_CLIST (clist), selected_row));
	gtk_clist_remove (GTK_CLIST (clist), selected_row);
}

static GtkWidget *
add_clist (GtkWidget *box, gint cols, gchar **titles)
{
	GtkWidget *sw;
	GtkWidget *clist;

	sw = e2_widget_add_sw (box, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC, TRUE, 0);
	clist = gtk_clist_new_with_titles (cols, titles);
	gtk_clist_column_titles_passive (GTK_CLIST (clist));
	gtk_clist_set_reorderable (GTK_CLIST (clist), TRUE);
	gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_BROWSE);
	gtk_container_add (GTK_CONTAINER (sw), clist);
	gtk_widget_show (clist);

	return clist;
}

static gboolean category_selected_cb (GtkTreeSelection *selection, GtkWidget *notebook)
{
	printd (DEBUG, "callback: category selected");
	GtkTreeIter iter;
	GtkTreeModel *model;
    gint page;

	if (gtk_tree_selection_get_selected (selection, &model, &iter))
		gtk_tree_model_get (model, &iter, 1, &page, -1);

	if (page >= 0)
	{
		page_last = page;
		page_last_iter = iter;
		gtk_notebook_set_page (GTK_NOTEBOOK (notebook), page);
		gtk_widget_grab_focus (treeview);
		return TRUE;
	}
	return FALSE;
}

static GtkWidget *add_page (GtkTreeIter *parent_iter, E2_ConfigDialogRuntime *rt,
	gchar *label_text)
{
	//add page to the treeview model
	GtkTreeIter child_iter;
	gtk_tree_store_append (rt->model, &child_iter, parent_iter);
	gint pages = gtk_notebook_get_n_pages (rt->notebook);
	gtk_tree_store_set (rt->model, &child_iter, 0, label_text,
		1, pages, -1);

	if (page_selected != NULL)
	{
		if ((g_str_equal (page_selected, label_text)) == TRUE)
		{
			page_last = pages;
		}
	}
	if (page_last == pages)
		page_last_iter = child_iter;

	//page vbox containing the headline and config widgets
	GtkWidget *vbox = gtk_vbox_new (FALSE, 5);
	g_object_set_data_full (G_OBJECT (vbox), "iter", gtk_tree_iter_copy (&child_iter),
		(void *) gtk_tree_iter_free);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);

	//page headline
	GtkWidget *frame = e2_widget_add_frame (vbox, FALSE, 0, NULL, FALSE);
	gchar *label_text2 = g_strconcat ("<b>", label_text, "</b>", NULL);
	GtkWidget *label = e2_widget_add_label (NULL, label_text2, 0.5, FALSE, 0);
	g_free (label_text2);
	gtk_misc_set_padding (GTK_MISC (label), 2, 2);
	gtk_container_add (GTK_CONTAINER (frame), label);
	gtk_notebook_append_page (rt->notebook, vbox, NULL);

	return vbox;
}

static GtkWidget *get_page (gchar *group, GtkTreeIter *iter, E2_ConfigDialogRuntime *rt)
{
	GtkWidget *box;
	if ((box = (GtkWidget *) g_hash_table_lookup (fast, group)) == NULL)
	{
		group = g_strdup (group);
		gchar *frame = strrchr (group, ':');
		if (frame != NULL)
		{
			*frame = '\0';
			GtkWidget *frame_box;
			if ((frame_box = (GtkWidget *) g_hash_table_lookup (fast, group)) == NULL)
				frame_box = get_page (group, iter, rt);
			gchar *frame_title = e2_utils_str_stretch (frame + 1);
			GtkWidget *frame_widget = gtk_frame_new (frame_title);
			g_free (frame_title);
			GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
			gtk_container_add (GTK_CONTAINER (frame_widget), vbox);
			gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
			gtk_box_pack_start (GTK_BOX (frame_box), frame_widget, FALSE, FALSE, 0);
			box = vbox;
			*frame = ':';
		} else
		{
			gchar *child = strrchr (group, '.');
			if (child != NULL)
			{
				*child = '\0';
				box = get_page (group, iter, rt);
				GtkTreeIter *iter2;
				iter2 = g_object_get_data (G_OBJECT (box), "iter");
				if (iter2)
					box = add_page (iter2, rt, child + 1);
				*child = '.';
			} else 
				box = add_page (iter, rt, group);
		}
		g_hash_table_insert (fast, group, box);
	}
	return box;
}

void create_config_dialog (gchar *page)
{
	printd (DEBUG, "create config dialog (%s)", page);

	E2_ConfigDialogRuntime *rt = g_malloc (sizeof (E2_ConfigDialogRuntime));
	
	gchar *titles[3];
	gchar *buf[3];
	GList *tmp;
	gint i;

	added_plugins = removed_plugins = NULL;

	//check if there is already a config dialog opened
	if (config_dialog != NULL)
	{
		gtk_window_present (GTK_WINDOW (config_dialog));
		return;
	}

	//find out current page to display
	if ((page == NULL) || (*page == '\0'))
	{
		if (!page_last)
			page = "general";
		else
			page = NULL;
	}
	page_selected = page;

	config_dialog = e2_dialog_create (NULL, NULL, "configuration", response_cb, rt);

	e2_option_connect (config_dialog, FALSE);

	g_signal_connect (GTK_OBJECT (config_dialog), "delete_event",
		G_CALLBACK (e2_cd_cancel_cb), NULL);
	g_signal_connect_after (GTK_OBJECT (config_dialog), "key_press_event",
		G_CALLBACK (key_press_cb), NULL);
	
	//the main hpane - contains the category treeview and the frame
	GtkWidget *hpane = gtk_hpaned_new ();
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (config_dialog)->vbox), hpane);
	
	//scrolled window for the category treeview
	GtkWidget *sw = e2_widget_get_sw (GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_widget_set_usize (sw, 200, 0);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_IN);
	gtk_paned_pack1 (GTK_PANED (hpane), sw, FALSE, TRUE);

	rt->model = gtk_tree_store_new (2, G_TYPE_STRING, G_TYPE_INT);

	GtkTreeIter iter;
	gtk_tree_store_append (rt->model, &iter, NULL);
	gtk_tree_store_set (rt->model, &iter, 0, "old options", 1, -1, -1);

	//tree view
	treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (rt->model));
	g_object_unref (rt->model);
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW
		(treeview));
	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);

	GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
	g_object_set (renderer, "xalign", 0.0, NULL);

	gint col_offset = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW
		(treeview), -1, "categories", renderer, "text", 0, NULL);
	GtkTreeViewColumn *column = gtk_tree_view_get_column (GTK_TREE_VIEW
		(treeview), col_offset - 1);
	gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), FALSE);
	gtk_container_add (GTK_CONTAINER (sw), treeview);

	//the notebook
	rt->notebook = GTK_NOTEBOOK (gtk_notebook_new ());
	gtk_notebook_set_show_tabs (rt->notebook, FALSE);
	gtk_notebook_set_show_border (rt->notebook, FALSE);
	sw = e2_widget_get_sw (GTK_POLICY_NEVER, GTK_POLICY_NEVER);
	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw),
		GTK_WIDGET (rt->notebook));
	gtk_viewport_set_shadow_type (GTK_VIEWPORT (GTK_BIN (sw)->child),
		GTK_SHADOW_NONE);
	gtk_paned_pack2 (GTK_PANED (hpane), sw, TRUE, TRUE);

	g_signal_connect (GTK_OBJECT (treeview), "key_press_event",
		G_CALLBACK (key_press_cb), NULL);
	g_signal_connect (G_OBJECT (selection), "changed",
		G_CALLBACK (category_selected_cb), rt->notebook);
	g_signal_connect (G_OBJECT (config_dialog), "show", G_CALLBACK (show_cb), sw);
	
	/*** old options ***/

	// Columns Tab
	GtkWidget *vbox;
	vbox = add_page (&iter, rt, _("Columns"));

	for (i = 0; i < MAX_COLUMNS; i++)
	{
		column_checks[i] = e2_button_add_toggle (vbox, all_columns[i].title,
				     0, FALSE, 0, NULL, NULL, TRUE);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (column_checks[i]),
				                         all_columns[i].is_visible);
	}
	gtk_widget_set_sensitive (column_checks[FILENAME], FALSE);

	// Filetypes Tab 
	vbox = add_page (&iter, rt ,_("Filetypes"));
	
	titles[0] = _("Extensions");
	titles[1] = _("Description");
	filetypes_clist = add_clist (vbox, 2, titles);

	for (tmp = cfg.filetypes; tmp != NULL; tmp = tmp->next)
	{
		FileType *ft = tmp->data;

		buf[0] = ft->extensions;
		buf[1] = ft->description;

		i = gtk_clist_append (GTK_CLIST (filetypes_clist), buf);
		gtk_clist_set_row_data (GTK_CLIST (filetypes_clist), i,
				                   g_memdup (ft, sizeof (FileType)));
	}

	GtkWidget *hbox;
	hbox = add_hbox (vbox, TRUE, 0, FALSE, 5);
	e2_button_add (hbox, FALSE, 5, "_up", GTK_STOCK_GO_UP, NULL,
		G_CALLBACK (clist_up_cb), filetypes_clist);
	e2_button_add (hbox, FALSE, 5, "_down", GTK_STOCK_GO_DOWN, NULL,
		G_CALLBACK (clist_down_cb), filetypes_clist);
	add_button (hbox, _("Add"), TRUE, 5, (GtkSignalFunc) add_filetype_cb, filetypes_clist);
	add_button (hbox, _("Edit"), TRUE, 5, (GtkSignalFunc) edit_filetype_cb, filetypes_clist);
	add_button (hbox, _("Remove"), TRUE, 5, (GtkSignalFunc) clist_remove_cb, filetypes_clist);

	// Plugins Tab
	vbox = add_page (&iter, rt, _("Plugins"));

	titles[0] = _("Name");
	titles[1] = _("Show in Menu");
	plugins_clist = add_clist (vbox, 2, titles);
	gtk_clist_set_column_width (GTK_CLIST (plugins_clist), 0, 200);

	for (tmp = cfg.plugins; tmp != NULL; tmp = tmp->next)
	{
		Plugin *plugin = tmp->data;

		buf[0] = plugin->name;
		buf[1] = (plugin->show_in_menu ? _("YES") : _("NO"));
		i = gtk_clist_append (GTK_CLIST (plugins_clist), buf);
		gtk_clist_set_row_data (GTK_CLIST (plugins_clist), i,
				                   g_memdup (plugin, sizeof (Plugin)));
	}

	hbox = add_hbox (vbox, TRUE, 0, FALSE, 5);
	e2_button_add (hbox, FALSE, 5, "_up", GTK_STOCK_GO_UP, NULL,
		G_CALLBACK (clist_up_cb), plugins_clist);
	e2_button_add (hbox, FALSE, 5, "_down", GTK_STOCK_GO_DOWN, NULL,
		G_CALLBACK (clist_down_cb), plugins_clist);
	add_button (hbox, _("Add"), TRUE, 5, (GtkSignalFunc) add_plugin_cb, plugins_clist);
	add_button (hbox, _("Info"), TRUE, 5, (GtkSignalFunc) plugin_info_cb, plugins_clist);
	add_button (hbox, _("Remove"), TRUE, 5, (GtkSignalFunc) remove_plugin_cb, plugins_clist);

	/*** new options ***/

	gtk_tree_store_append (rt->model, &iter, NULL);
	gtk_tree_store_set (rt->model, &iter, 0, "new options", 1, -1, -1);
	
	fast = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);	
	
	for (i = 0; i < options_array->len; i++)
	{
		E2_OptionSet *set = g_ptr_array_index (options_array, i);
		
		gchar *size_group_key = g_strconcat (set->group, "_sizegroup", NULL);
		GtkSizeGroup *size_group = (GtkSizeGroup *) g_hash_table_lookup (fast, size_group_key);
		//insert size group for page
		if (!GTK_IS_SIZE_GROUP (size_group))
		{
			size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
			g_hash_table_insert (fast, g_strdup (size_group_key), size_group);
		}
		GtkWidget *box = get_page (set->group, &iter, rt);
		switch (set->type)
		{
			case E2_OPTION_TYPE_TREE:
				e2_option_tree_add_widget (config_dialog, box, set);
				break;
			case E2_OPTION_TYPE_BOOL:
				e2_option_bool_add_dialog_widget (config_dialog, box, size_group, set);
				break;
			case E2_OPTION_TYPE_STR:
			{
				GtkWidget *hbox = gtk_hbox_new (FALSE, 5);
				gtk_box_pack_start (GTK_BOX (box), hbox, TRUE, TRUE, 5);
				GtkWidget *label = e2_widget_add_label (hbox, set->desc, 0.0, FALSE, 0);
				gtk_size_group_add_widget (size_group, label);
				set->widget = e2_widget_add_entry (hbox, set->sdef, TRUE, FALSE);
				e2_widget_set_tooltip (NULL, set->widget, set->tip);
			}
				break;
			case E2_OPTION_TYPE_INT:
				e2_option_int_add_widget (config_dialog, box, size_group, set);
				break;
			case E2_OPTION_TYPE_SEL:
				e2_option_sel_add_widget (config_dialog, box, size_group, set);
				break;
			case E2_OPTION_TYPE_FONT:
			{
				//hbox for option label, font entry and font dialog button
				GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
				gtk_box_pack_start (GTK_BOX (box), hbox, TRUE, TRUE, 0);
				//option label
				gchar *label_text = g_strconcat (set->desc, ": ", NULL);
				GtkWidget *label = e2_widget_add_label (hbox, label_text, 0.0, FALSE, 0);
				g_free (label_text);
				gtk_size_group_add_widget (size_group, label);
				//font entry for font name
				set->widget = e2_widget_add_entry (hbox, NULL, TRUE, FALSE);
				e2_widget_set_tooltip (NULL, set->widget, set->tip);
				//button for font select dialog
				e2_button_add (hbox, FALSE, 0, "change", GTK_STOCK_SELECT_FONT,
					"press button to open a font select dialog", font_select_cb, set);
				//label for font example
				label = e2_widget_add_label (box, "", 0.0, FALSE, 0);
				gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
				//react on changes to the font name
				g_signal_connect (GTK_EDITABLE (set->widget), "changed",
					G_CALLBACK (font_entry_cb), label);
				//set entry after signal connect so the example label gets
				//some contents it's empty at the beginning
				gtk_entry_set_text (GTK_ENTRY (set->widget), set->sdef);
			}
				break;
			case E2_OPTION_TYPE_COLOR:
			{
				//hbox for option label, color entry and color dialog button
				GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
				gtk_box_pack_start (GTK_BOX (box), hbox, TRUE, TRUE, 0);
				//option label
				gchar *label_text = g_strconcat (set->desc, ": ", NULL);
				GtkWidget *label = e2_widget_add_label (hbox, label_text, 0.0, FALSE, 0);
				g_free (label_text);
				gtk_size_group_add_widget (size_group, label);
				//color entry for color names/color hex values
				set->widget = e2_widget_add_entry (hbox, NULL, TRUE, FALSE);
				e2_widget_set_tooltip (NULL, set->widget, set->tip);
				//button for color select dialog
				e2_button_add (hbox, FALSE, 0, "change", GTK_STOCK_SELECT_COLOR,
					"press button to open a color select dialog", color_select_cb, set);
				//label for color example
				label = e2_widget_add_label (box, "", 0.0, FALSE, 0);
				gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
				//react on changes to the color name/color hex value
				g_signal_connect (GTK_EDITABLE (set->widget), "changed", G_CALLBACK (color_entry_cb), label);
				//set entry after signal connect so the example label gets its
				//some contents, it's empty at the beginning
				gtk_entry_set_text (GTK_ENTRY (set->widget), set->sdef);
			}
				break;
			default:
				break;
		}
		g_free (size_group_key);
	}
	for (i = 0; i < options_array->len; i++)
	{
		E2_OptionSet *set = g_ptr_array_index (options_array, i);
		if (set->depends != NULL)
		{
			if (set->depends[0] == '!')
			{
				E2_OptionSet *opt = e2_option_get (set->depends + 1);
				if ((opt != NULL) && (opt->type == E2_OPTION_TYPE_BOOL))
				{
					g_signal_connect (GTK_OBJECT (opt->widget),
						"toggled", G_CALLBACK (depends_cb), set->widget);
					gtk_widget_set_sensitive (set->widget,
						!GTK_TOGGLE_BUTTON (opt->widget)->active);
				}
			} else
			{
				E2_OptionSet *opt = e2_option_get (set->depends);
				if ((opt != NULL) && (opt->type == E2_OPTION_TYPE_BOOL))
				{
					g_signal_connect (GTK_OBJECT (opt->widget),
						"toggled", G_CALLBACK (depends_cb_), set->widget);
					gtk_widget_set_sensitive (set->widget,
						GTK_TOGGLE_BUTTON (opt->widget)->active);
				}
			}
		}
	}
	
	//clean up
	g_hash_table_destroy (fast);
	
	//last preparation for the category tree view
	//ie: expand, select last/first row
	gtk_tree_view_expand_all (GTK_TREE_VIEW (treeview));
	if (page_last >= 0)
	{

		GtkTreePath *path;
		path = gtk_tree_model_get_path (GTK_TREE_MODEL (rt->model), &page_last_iter);
		GtkTreeViewColumn *col = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview),
			0);
		gtk_tree_view_set_cursor  (GTK_TREE_VIEW (treeview), path, col, FALSE);
		gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (treeview), path, col, TRUE, 0.5, 0.5);
		gtk_tree_path_free (path);
	}
	e2_dialog_show (config_dialog, app.main_window,
		E2_BUTTON_CANCEL | E2_BUTTON_APPLY | E2_BUTTON_OK, 0);
	gtk_notebook_set_page (rt->notebook, page_last);
}

