/* $Id: e2_context_menu.c,v 1.11 2004/06/15 22:48:14 cairo Exp $
 * 
 * Copyright (C) 2003-2004 tooar <tooar@gmx.net>
 * Copyright (C) 1999 Michael Clark.
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * 09-20-2000: Modified by Aurelien Gateau
 *             Added code for Named actions for filetypes
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "emelfm2.h"
#include "e2_context_menu.h"
#include "debug.h"
#include "e2_option.h"
#include "e2_action.h"
#include "e2_utils.h"
#include "menu_actions_cb.h"
#include "e2_bookmark.h"
#include "e2_tree.h"

// Right Click Menu Callbacks
static void choose_action_cb (GtkWidget *widget)
{
	exec_filetype_action ((gchar *) gtk_widget_get_name (widget));
}

/*
 * Filter Menu Callbacks
 */
static void
filename_filter_cb(GtkWidget *widget, FileView *view)
{
	create_filename_filter_dialog(view);
}

static void
size_filter_cb(GtkWidget *widget, FileView *view)
{
	create_size_filter_dialog(view);
}

static void
date_filter_cb(GtkWidget *widget, FileView *view)
{
	create_date_filter_dialog(view);
}

static void
filter_dirs_cb(GtkWidget *widget, FileView *view)
{
	view->filter_directories = GTK_CHECK_MENU_ITEM(widget)->active;
	refresh_list(view);
}

static void
remove_filters_cb(GtkWidget *widget, FileView *view)
{
	remove_filters(view);
}

static void sync_cb (GtkWidget *wid, FileView *view)
{
	if (view != curr_view)
		e2_panes_switch ();
	e2_pane_change_dir (app.pane_other, app.pane_current->path);
}

GtkWidget *create_filelist_menu_bar (FileView *view)
{
	GtkWidget *menu_bar = gtk_menu_bar_new();
	
	view->hidden_button = e2_menu_add_check (menu_bar, "show hidden",
		view->show_hidden, toggle_show_hidden_cb, view);

	// Filter Menu
	GtkWidget *menu = gtk_menu_new();
	add_menu_item(menu, _("Filename Filter"),
		G_CALLBACK (filename_filter_cb), view);
	add_menu_item(menu, _("Size Filter"),
		G_CALLBACK (size_filter_cb), view);
	add_menu_item(menu, _("Date Filter"),
		G_CALLBACK (date_filter_cb), view);
	e2_menu_add_separator(menu);
	e2_menu_add_check(menu, _("Filter Directories"), FALSE,
		G_CALLBACK (filter_dirs_cb), view);
	e2_menu_add_separator(menu);
	add_menu_item(menu, _("Remove All Filters"),
		G_CALLBACK (remove_filters_cb), view);
	view->filter_menu_item = add_submenu (menu_bar, _("_Filters"), menu);
	if (view->filename_filter.active
			|| view->size_filter.active
			|| view->date_filter.active)
	{
		set_filter_menu_active(view);
	}

	add_menu_item2 (menu_bar, _("select all"), NULL,
		fileview_select_all2, view);
	add_menu_item2 (menu_bar, view == &app.left_view ? _("&gt;") : _("&lt;"), NULL,
		sync_cb, view);

	return menu_bar;
}

void create_plugins_menu (GtkWidget *menu)
{
	GList *tmp;

	for (tmp = cfg.plugins; tmp != NULL; tmp = tmp->next)
	{
		Plugin *p = tmp->data;
		if (p->show_in_menu)
			add_menu_item(menu, p->name, G_CALLBACK (do_plugin_action), p);
	}
}

static GtkWidget *create_actions_menu (GtkWidget *menu, gchar *actions)
{
	gchar *s, *sep, *free_this;
	GtkWidget *menu_item;

	actions = free_this = g_strdup(actions);

	while ((s = strchr(actions, ',')) != NULL)
	{
		*s++ = '\0';
		if ((sep = strchr(actions, '@')) != NULL)
		{
			*sep++ = '\0';
			menu_item = add_menu_item(menu, actions, G_CALLBACK (choose_action_cb), NULL);
			gtk_widget_set_name(GTK_WIDGET(menu_item), sep);
		}
		else
		{
			menu_item = add_menu_item(menu, actions, G_CALLBACK (choose_action_cb), NULL);
			gtk_widget_set_name(GTK_WIDGET(menu_item), actions);
		}
		actions = s;
	}

	//get the last action
	if ((sep = strchr(actions, '@')) != NULL)
	{
		*sep++ = '\0';
		menu_item = add_menu_item(menu, actions, G_CALLBACK (choose_action_cb), NULL);
		gtk_widget_set_name(GTK_WIDGET(menu_item), sep);
	}
	else
	{
		menu_item = add_menu_item(menu, actions, G_CALLBACK (choose_action_cb), NULL);
		gtk_widget_set_name(GTK_WIDGET(menu_item), actions);
	}
	g_free(free_this);
	return menu;
}

static void set_filetype_menus (GtkWidget *menu, FileInfo *info)
{
	gchar *ext, *actions; 

	if (info == NULL)
		return;
	ext = strchr(info->filename, '.');
	if ((is_dir (info)) == TRUE)
		ext = ".<directory>";
	if ((is_executable (info)) == TRUE)
		ext = ".<executable>";

	if (ext != NULL)
	{
		//check all possible extensions for a matching filetype
		do
		{
			//remove leading dot "."
			ext++;
			actions = get_actions_for_ext(ext);
			if (actions != NULL)
			{
				create_actions_menu (menu, actions);
				g_free (actions);
				return;
			}
		} while ((ext = strchr(ext, '.')) != NULL);
	}
}

/*
static gboolean item_click_cb (GtkWidget *widget, GdkEventButton *event, E2_ActionRuntime *rt)
{
	printd (DEBUG, "item_click_cb (widget:_,event:_,rt:_)");
	if (event->button == 1)
	{
		e2_action_run (widget, rt);
		return TRUE;
	}
	return FALSE;
}
*/

  /******************/
 /***** public *****/
/******************/

void e2_menu_add_bookmarks (E2_Action *action, E2_OptionSet *set,
	GtkWidget *menu, GtkTreeIter *iter)
{
	do
	{
		gchar *_name, *icon, *_tip, *_path;
		gtk_tree_model_get (set->ex.tree.model, iter, 0, &_name, 1, &icon, 2, &_tip, 3, &_path, -1);
		gchar *name = e2_utils_replace_env_vars (_name);
		gchar *tip = e2_utils_replace_env_vars (_tip);
		gchar *path = e2_utils_replace_env_vars (_path);
		g_free (_name);
		g_free (_tip);
		g_free (_path);

		GtkWidget *item;
		gboolean child = gtk_tree_model_iter_has_child (set->ex.tree.model, iter);
		if (child)
		{
			GtkWidget *menu2 = gtk_menu_new ();
			GtkTreeIter iter2;
			if (gtk_tree_model_iter_children (set->ex.tree.model, &iter2, iter))
				e2_menu_add_bookmarks (action, set, menu2, &iter2);
			item = e2_menu_add (menu, name, icon, tip, NULL, NULL);
			gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu2);
			g_free (path);
		} else
		{
			item = e2_menu_add (menu, name, icon, tip, NULL, NULL);
		}
		g_signal_connect_data (item, "button-press-event", G_CALLBACK (e2_bookmark_click_cb),
			gtk_tree_model_get_path (set->ex.tree.model, iter), (GClosureNotify) gtk_tree_path_free, 0);
		//gtk sillyness fix, if we don't use connect_after for the activate signal
		//of the menuitem, the button-press-event will never occur
		if (!child)
		{
			E2_ActionRuntime *art = e2_action_pack_runtime (action, path, g_free);
			g_signal_connect_after (item, "activate", G_CALLBACK (e2_action_run), art);
			g_object_set_data_full (G_OBJECT (item), "free-callback-data-bookmark", art,
				(GDestroyNotify) e2_action_free_runtime);
		}
		g_object_set_data (G_OBJECT (item), "bookmark-path", path);
		g_free (name);
		g_free (icon);
		g_free (tip);
	} while ((gtk_tree_model_iter_next (set->ex.tree.model, iter)) == TRUE);
}

void e2_menu_add_item (GtkWidget *menu, GtkTreeModel *model, GtkTreeIter *iter, gint level, FileInfo *info, gint mtype)
{
	do
	{
		gchar *icon, *str_data, *type, *arg;
		gboolean _mtype;
		switch (mtype)
		{
			case 1:
				gtk_tree_model_get (model, iter, 0, &str_data, 4, &type, 1, &icon, 2, &_mtype, 5, &arg, -1);
				break;
			case 2:
				gtk_tree_model_get (model, iter, 0, &str_data, 4, &type, 1, &icon, 3, &_mtype, 5, &arg, -1);
				break;
			default:
				gtk_tree_model_get (model, iter, 0, &str_data, 4, &type, 1, &icon, 5, &arg, -1);
				break;
		}
		if ((mtype != 0) && (_mtype != TRUE))
		{
			if (gtk_tree_model_iter_has_child (model, iter))
			{
				GtkTreeIter iter2;
				gtk_tree_model_iter_children (model, &iter2, iter);
				e2_menu_add_item (menu, model, &iter2, level, info, mtype);
			}
		} else
		{
			E2_Action *action = e2_action_get (type);
			if (action != NULL) 
			switch (action->type)
			{
				case E2_ACTION_TYPE_SUBMENU:
				{
					GtkWidget *item = add_menu_item2 (menu, str_data, icon, NULL, NULL);
					GtkWidget *menu2 = gtk_menu_new ();
					GtkTreeIter iter2;
					gtk_tree_model_iter_children (model, &iter2, iter);
					e2_menu_add_item (menu2, model, &iter2, level + 1, info, mtype);
					gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu2);
					break;
				}
				case E2_ACTION_TYPE_BOOKMARKS:
				{
					E2_OptionSet *set = e2_option_get ("bookmarks");
					GtkTreeIter iter;
					if ((arg != NULL) && (*arg != '\0'))
					{
						if (e2_tree_find_iter_from_str (set->ex.tree.model, 0, arg, &iter, TRUE))
						{
							GtkTreeIter iter2;
							if (gtk_tree_model_iter_children (set->ex.tree.model, &iter2, &iter))
								e2_menu_add_bookmarks (action, set, menu, &iter2);

						}
					} else if (gtk_tree_model_get_iter_first (set->ex.tree.model, &iter))
							e2_menu_add_bookmarks (action, set, menu, &iter);
				}
					break;
				case E2_ACTION_TYPE_FILE_ACTIONS:
					set_filetype_menus (menu, info);
					break;
				case E2_ACTION_TYPE_PLUGINS:
					create_plugins_menu (menu);
					break; 
				case E2_ACTION_TYPE_SEPARATOR:
					e2_menu_add_separator (menu);
					break;
				case E2_ACTION_TYPE_TEAR_OFF_MENU:
					add_menu_tear_off (menu);
					break;
				case E2_ACTION_TYPE_TOGGLE:
				case E2_ACTION_TYPE_ITEM:
					{
						E2_ActionRuntime *actionrt = e2_action_pack_runtime (action, arg, g_free);
						GtkWidget *item;
						if (action->type == E2_ACTION_TYPE_TOGGLE)
							item = e2_menu_add_check (menu, str_data, e2_option_bool_get
								(action->data2), e2_action_run, actionrt);
						else
							item = add_menu_item2 (menu, str_data, icon, e2_action_run,
								actionrt);
						g_object_set_data_full (G_OBJECT (item), "free-callback-data", actionrt,
							(GDestroyNotify) e2_action_free_runtime);
					}
					break;
				default:
					break;
			} 
		}
		//cleanup
		g_free (icon);
		g_free (str_data);
		g_free (type);
//		g_free (arg);
	} while ((gtk_tree_model_iter_next (model, iter)) == TRUE);
}

static void destroy_menu_cb (GtkWidget *menu_shell, GtkWidget *menu)
{
	printd (DEBUG, "destroy_menu_cb (menu_shell:,menu:)");
	gtk_container_foreach (GTK_CONTAINER (menu), (void *) gtk_widget_destroy, NULL);
	gtk_widget_destroy (menu);
	reenable_refresh();
	app.context_menu = NULL;
}

void e2_menu_show (guint button, guint32 time, gint type)
{
	FileInfo *info;
	
	disable_refresh();
	if (curr_view->tagged != NULL)
		info = curr_view->tagged->data;
	else
		info = gtk_clist_get_row_data(GTK_CLIST(curr_view->clist), curr_view->row);
	
	E2_OptionSet *set = e2_option_get ("context-menu");
	app.context_menu = gtk_menu_new ();
	GtkTreeIter iter;
	if ((gtk_tree_model_get_iter_first (set->ex.tree.model, &iter)) == TRUE)
		e2_menu_add_item (app.context_menu, set->ex.tree.model, &iter, 0, info, type);
	
	g_signal_connect (G_OBJECT (app.context_menu), "selection-done",
		G_CALLBACK (destroy_menu_cb), app.context_menu);
	gtk_menu_popup (GTK_MENU (app.context_menu), NULL, NULL, NULL, NULL, button, time);
}

static void show_menu_action (gpointer data, gchar *arg, GtkWidget *origin)
{
	g_strstrip (arg);
	gchar *str_down = g_utf8_strdown (arg, -1);
	if (g_str_equal (str_down, "shift"))
		e2_menu_show (0, 0, 1);
	else if (g_str_equal (str_down, "ctrl"))
		e2_menu_show (0, 0, 2);
	else
		e2_menu_show (0, 0, 0);
	g_free (str_down);
}

void e2_context_menu_init ()
{
	_e2_action_register ("panes.show menu", E2_ACTION_TYPE_ITEM,
		show_menu_action, NULL, TRUE, E2_ACTION_EXCLUDE_MENU, NULL);
}
