/* $Id: callbacks.c,v 1.8 2004/08/14 12:01:08 cairo Exp $
 *
 * Copyright (C) 2003-2004 tooar <tooar@gmx.net>
 * Copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */


#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include "emelfm2.h"
#include "debug.h"
#include "e2_dialog.h"
#include "e2_command.h"
#include "e2_option.h"
#include "e2_button.h"
#include "e2_bookmark.h"
#include "e2_action.h"
#include "e2_utils.h"
#include "e2_output.h"
#include "e2_pane.h"

void copy_cb (GtkWidget *widget)
{
	FileInfo *info;
	GList *tmp, *base;
	gchar src[PATH_MAX+NAME_MAX], dest[PATH_MAX+NAME_MAX];
	gboolean check = e2_option_bool_get ("confirm-overwrite");
	gint result;

	e2_widget_set_cursor (GDK_WATCH);
	disable_refresh();
	base = tmp = get_selection(curr_view);

	gtk_widget_set_sensitive(app.main_window, FALSE);
	for (; tmp != NULL; tmp = tmp->next)
	{
		info = tmp->data;
		g_snprintf(src, sizeof(src), "%s/%s", curr_view->dir, info->filename);
		g_snprintf(dest, sizeof(dest), "%s/%s", other_view->dir, info->filename);
		if (access(dest, F_OK) == 0)
		{
			if (is_dir (info))
				g_snprintf(dest, sizeof(dest), "%s", other_view->dir);
			if (check)
			{
				result = op_with_ow_check (src, dest, file_copy);
				if (result == YES_TO_ALL)
				  check = FALSE;
				else if (result == CANCEL)
				  break;
			} else
				file_copy (src, dest);
		}
		else
			file_copy(src, dest);
	}
	gtk_widget_set_sensitive(app.main_window, TRUE);

	
	g_list_free(base);
	refresh_list(other_view);
	refresh_list(other_view);
	e2_widget_set_cursor (GDK_LEFT_PTR);
	reenable_refresh();
}

void copy_as_cb (GtkWidget *widget)
{
	FileInfo *info;
	GList *tmp, *base;
	gchar label[NAME_MAX+32];
	gchar dest[PATH_MAX+NAME_MAX];
	gchar src[PATH_MAX+NAME_MAX];
	gchar *new_name;
	gboolean check = e2_option_bool_get ("confirm-overwrite");
	gint result;

	e2_widget_set_cursor (GDK_WATCH);
	gtk_widget_set_sensitive(app.main_window, FALSE);
	disable_refresh();
	base = tmp = get_selection(curr_view);

	for (; tmp != NULL; tmp = tmp->next)
	{
		info = tmp->data;
		strncpy(src, info->filename, sizeof(src));

		g_snprintf(label, sizeof(label), _("Enter new filename for: %s"), src);
		create_user_prompt(label, src, FALSE, &new_name);
		gtk_main();

		if (new_name != NULL)
		{
			g_snprintf(dest, sizeof(dest), "%s/%s", other_view->dir, new_name);
			g_snprintf(src, sizeof(src), "%s/%s", curr_view->dir, info->filename);
			if (access(dest, F_OK) == 0)
			{
				if (is_dir (info))
					g_snprintf(dest, sizeof(dest), "%s", other_view->dir);
				if (check)
				{
			  result = op_with_ow_check(src, dest, file_copy);
			  if (result == YES_TO_ALL)
			    check = FALSE;
			  else if (result == CANCEL)
			    break;
			  	}
			}
			else
			  file_copy(src, dest);
			g_free(new_name);
		}
	}

	g_list_free(base);
	gtk_widget_set_sensitive(app.main_window, TRUE);
	refresh_list(other_view);
	refresh_list(other_view);
	e2_widget_set_cursor (GDK_LEFT_PTR);
	reenable_refresh();
}

void move_cb (GtkWidget *widget)
{
	printd (DEBUG, "callback: move");
	FileInfo *info;
	GList *tmp, *base;
	gchar src[PATH_MAX+NAME_MAX], dest[PATH_MAX+NAME_MAX];
	gboolean check = e2_option_bool_get ("confirm-overwrite");
	gint result;


	e2_widget_set_cursor (GDK_WATCH);
	disable_refresh();
	gtk_widget_set_sensitive(app.main_window, FALSE);
	base = tmp = get_selection(curr_view);

	for (; tmp != NULL; tmp = tmp->next)
	{
		info = tmp->data;
		g_snprintf(src, sizeof(src), "%s/%s", curr_view->dir, info->filename);
		g_snprintf(dest, sizeof(dest), "%s/%s", other_view->dir, info->filename);
		if (check && (access(dest, F_OK) == 0))
		{
			result = op_with_ow_check(src, other_view->dir, file_move);
//			file_delete (src);
			if (result == YES_TO_ALL)
			  check = FALSE;
			else if (result == CANCEL)
			  break;
		} else
		{
			file_move (src, other_view->dir);
//			file_delete (src);
		}
	}
	g_list_free(base);

	refresh_list(other_view);
	refresh_list(other_view);
	gtk_clist_unselect_all(GTK_CLIST(curr_view->clist));
	e2_widget_set_cursor (GDK_LEFT_PTR);
	gtk_widget_set_sensitive(app.main_window, TRUE);
	reenable_refresh();
	gtk_widget_grab_focus (app.pane_current->focus_widget);
}

void move_as_cb (GtkWidget *widget)
{
	FileInfo *info;
	GList *tmp, *base;
	gchar label[NAME_MAX+32];
	gchar dest[PATH_MAX+NAME_MAX];
	gchar src[PATH_MAX+NAME_MAX];
	gchar *new_name;
	gboolean check = e2_option_bool_get ("confirm-overwrite");
	gint result;

	e2_widget_set_cursor (GDK_WATCH);
	gtk_widget_set_sensitive(app.main_window, FALSE);
	disable_refresh();
	base = tmp = get_selection(curr_view);

	for (; tmp != NULL; tmp = tmp->next)
	{
		info = tmp->data;
		strncpy(src, info->filename, sizeof(src));

		g_snprintf(label, sizeof(label), _("Enter new filename for: %s"), src);
		create_user_prompt(label, src, FALSE, &new_name);
		gtk_main();

		if (new_name != NULL)
		{
			g_snprintf(dest, sizeof(dest), "%s/%s", other_view->dir, new_name);
			g_snprintf(src, sizeof(src), "%s/%s", curr_view->dir, info->filename);
			if (check && (access(dest, F_OK) == 0))
			{
			  result = op_with_ow_check(src, dest, file_move);
			  if (result == YES_TO_ALL)
			    check = FALSE;
			  else if (result == CANCEL)
			    break;
			}
			else
			  file_move(src, dest);
			g_free(new_name);
		}
	}

	g_list_free(base);
	gtk_widget_set_sensitive(app.main_window, TRUE);
	refresh_list(other_view);
	refresh_list(other_view);
	gtk_clist_unselect_all(GTK_CLIST(curr_view->clist));
	e2_widget_set_cursor (GDK_LEFT_PTR);
	reenable_refresh();
}

void delete_cb (GtkWidget *widget)
{
	printd (DEBUG, "callback: delete");
	gboolean check = e2_option_bool_get ("confirm-delete");

	prepare_dialog ();
	GList *tmp, *base;
	base = tmp = get_selection(curr_view);

	for (; tmp != NULL; tmp = tmp->next)
	{
		FileInfo *info;
		info = tmp->data;
		gchar *filename = g_build_path (curr_view->dir, info->filename, NULL);
		if (check)
		{
			gchar *utf = _STR2UTF (info->filename);
			gchar *mesc = MESC (utf);
			g_free (utf);
			gchar *dialog_text = g_strconcat  ("are you sure you want to delete <b>",
				mesc, "</b>?", NULL);
			g_free (mesc);
			GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
				dialog_text, "confirm delete", NULL, NULL);
			gint result = e2_dialog_show (dialog, app.main_window,
				E2_BUTTON_CANCEL | E2_BUTTON_NO |
				E2_BUTTON_YES_TO_ALL | E2_BUTTON_YES,
				E2_DIALOG_RUN | E2_DIALOG_FREE);
			g_free (dialog_text);
			switch (result)
			{
				case GTK_RESPONSE_YES:
					printd (DEBUG, "callback: delete yes");
			  		file_delete (filename);
					break;
				//111 is yes to all
				case 111:
					printd (DEBUG, "callback: delete yes to all");
					file_delete(filename);
					check = FALSE;
					break;
				case GTK_RESPONSE_NO:
					printd (DEBUG, "callback: delete no");
					break;
				default:
					printd (DEBUG, "callback: delete cancel");
			}
			if (result == GTK_RESPONSE_CANCEL) break;
		}
		else
			file_delete(filename);
		
		g_free (filename);
	}
	g_list_free (base);
	postpare_dialog (TRUE, other_view);
	refresh_list(other_view);
}

void symlink_cb (GtkWidget *widget)
{
	FileInfo *info;
	gchar src[PATH_MAX+NAME_MAX], dest[PATH_MAX+NAME_MAX], rel[PATH_MAX+NAME_MAX];
	GList *tmp, *base;

	gboolean use_rel_links = e2_option_bool_get ("relative-symlinks");
	
	e2_widget_set_cursor (GDK_WATCH);
	disable_refresh();
	base = tmp = get_selection(curr_view);

	gchar *separator = "/";
	if (e2_option_bool_get ("transparent-dir-links"))
		separator = "";

	for (; tmp != NULL; tmp = tmp->next)
	{
		info = tmp->data;
		g_snprintf(src, sizeof(src), "%s%s%s", curr_view->dir, separator, info->filename);
		g_snprintf(dest, sizeof(dest),  "%s%s%s", other_view->dir, separator, info->filename);

		if (TRUE == use_rel_links) 
		{
			e2_utils_rel_path (src, dest, rel);
			strncpy(src, rel, sizeof(src));
		}

		if (file_symlink(src, dest) == -1)
			status_errno();
	}
	g_list_free(base);

	refresh_list(other_view);
	refresh_list(other_view);
	e2_widget_set_cursor (GDK_LEFT_PTR);
	reenable_refresh();
}

void symlink_as_cb (GtkWidget *widget)
{
	FileInfo *info;
	gchar label[NAME_MAX+32];
	gchar *new_name;
	gchar src[PATH_MAX+NAME_MAX], dest[PATH_MAX+NAME_MAX], rel[PATH_MAX+NAME_MAX];
	GList *tmp, *base;

	gboolean use_rel_links = e2_option_bool_get ("relative-symlinks");

	e2_widget_set_cursor (GDK_WATCH);
	gtk_widget_set_sensitive(app.main_window, FALSE);
	disable_refresh();
	base = tmp = get_selection(curr_view);

	gchar *separator = "/";
	if (e2_option_bool_get ("transparent-dir-links"))
		separator = "";

	for (; tmp != NULL; tmp = tmp->next)
	{
		info = tmp->data;
		strncpy(src, info->filename, sizeof(src));

		g_snprintf(label, sizeof(label), _("Enter name for symlink to: %s"), src);
		create_user_prompt(label, src, FALSE, &new_name);
		gtk_main();

		if (new_name != NULL)
		{
			g_snprintf(src, sizeof(src), "%s%s%s", curr_view->dir, separator, info->filename);
			g_snprintf(dest, sizeof(dest),  "%s%s%s", other_view->dir, separator, new_name);


			if (TRUE == use_rel_links)
			{
				e2_utils_rel_path (src, dest, rel);
				strncpy(src, rel, sizeof(src));
			}

			if (file_symlink(src, dest) == -1)
				status_errno();
			g_free(new_name);
		}
	}

	g_list_free(base);
	gtk_widget_set_sensitive(app.main_window, TRUE);
	refresh_list(other_view);
	refresh_list(other_view);
	e2_widget_set_cursor (GDK_LEFT_PTR);
	reenable_refresh();
}

void permissions_cb (GtkWidget *widget)
{
	FileInfo *info;
	GList *tmp, *base;
	gchar path[PATH_MAX+NAME_MAX];

	guint answer;
	gboolean recurse;
	GString *mode;

	e2_widget_set_cursor (GDK_WATCH);
	gtk_widget_set_sensitive(app.main_window, FALSE);
	disable_refresh();
	base = tmp = get_selection(curr_view);

	for (; tmp != NULL; tmp = tmp->next)
	{
		info = tmp->data;
		create_permissions_dialog(info, &answer, &recurse, &mode);
		gtk_main();

		if (answer == OK)
		{
			g_snprintf(path, sizeof(path), "%s/%s", curr_view->dir, info->filename);
			file_chmod(path, mode->str, recurse);
			g_string_free(mode, TRUE);
		}
		else if (answer == APPLY_TO_ALL)
		{
			for (; tmp != NULL; tmp = tmp->next)
			{
			  info = tmp->data;
			  g_snprintf(path, sizeof(path), "%s/%s", curr_view->dir, info->filename);
			  file_chmod(path, mode->str, recurse);
			}
			g_string_free(mode, TRUE);
			break;
		}
		else if (answer == CANCEL)
			break;
	}

	g_list_free(base);
	gtk_widget_set_sensitive(app.main_window, TRUE);
	refresh_list(curr_view);
	e2_widget_set_cursor (GDK_LEFT_PTR);
	reenable_refresh();
}

void ownership_cb (GtkWidget *widget)
{
	FileInfo *info;
	GList *tmp, *base;
	gchar path[PATH_MAX+NAME_MAX];

	uid_t owner_id;
	gid_t group_id;
	gboolean recurse;
	guint answer;

	e2_widget_set_cursor (GDK_WATCH);
	gtk_widget_set_sensitive(app.main_window, FALSE);
	disable_refresh();
	base = tmp = get_selection(curr_view);

	for (; tmp != NULL; tmp = tmp->next)
	{
		info = tmp->data;
		create_ownership_dialog(info, &answer, &owner_id, &group_id, &recurse);
		gtk_main();

		if (answer == OK)
		{
			g_snprintf(path, sizeof(path), "%s/%s", curr_view->dir, info->filename);
			file_chown(path, owner_id, group_id, recurse);
		}
		else if (answer == APPLY_TO_ALL)
		{
			for (; tmp != NULL; tmp = tmp->next)
			{
			  info = tmp->data;
			  g_snprintf(path, sizeof(path), "%s/%s", curr_view->dir, info->filename);
			  file_chown(path, owner_id, group_id, recurse);
			}
			break;
		}
		else if (answer == CANCEL)
			break;
	}

	g_list_free(base);
	gtk_widget_set_sensitive(app.main_window, TRUE);
	refresh_list(curr_view);
	e2_widget_set_cursor (GDK_LEFT_PTR);
	reenable_refresh();
}

void file_info_cb (GtkWidget *widget)
{
	FileInfo *info;
	GList *tmp, *base;

	prepare_dialog ();

	base = tmp = get_selection (curr_view);

	for (; tmp != NULL; tmp = tmp->next)
	{
		info = tmp->data;
		create_file_info_dialog(info);
		gtk_main();
	}

	g_list_free (tmp);
	postpare_dialog (FALSE, NULL);
	focus_row(curr_view, curr_view->row, FALSE, FALSE, TRUE);
}

void rename_cb (GtkWidget *widget)
{
	printd (DEBUG, "callback: rename");
	FileInfo *info;
	GList *tmp, *base, *new_list = NULL;
	gboolean check = e2_option_bool_get ("confirm-overwrite");

	prepare_dialog ();

	base = tmp = get_selection(curr_view);

	for (; tmp != NULL; tmp = tmp->next)
	{
		info = tmp->data;
		gchar src[PATH_MAX+NAME_MAX];
		gchar dest[PATH_MAX+NAME_MAX];
		strncpy (src, info->filename, sizeof (src));
		gchar *src_ = _STR2UTF (src);

		gchar dialog_text[NAME_MAX+32];
		gchar *mesc = MESC (src_);
		g_snprintf (dialog_text, sizeof (dialog_text), 
			"enter new filename for <b>%s</b>", mesc);
		g_free (mesc);
		
		GtkWidget *dialog, *entry;
		dialog = create_user_prompt2 (dialog_text, src_, TRUE, &entry);
		gint result = e2_dialog_show (dialog, app.main_window, 0,
				E2_DIALOG_RUN);
		switch (result)
		{
			case GTK_RESPONSE_OK:
				printd (DEBUG, "callback: rename ok");
				gchar *new_name = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
				if ((new_name != NULL) && (!g_str_equal (new_name, info->filename)))
				{
					new_name = _UTF2STR (new_name);
					g_snprintf (dest, sizeof (dest), "%s/%s", curr_view->dir, new_name);
					g_snprintf (src, sizeof (src), "%s/%s", curr_view->dir, info->filename);
					if (check && (access (dest, F_OK) == 0))
					{
						result = op_with_ow_check(src, dest, file_move);
						if (result == YES_TO_ALL)
					    	check = FALSE;
					}
					else
					  file_move(src, dest);

					new_list = g_list_append (new_list, new_name);
				}
				break;
			default:
				printd (DEBUG, "callback: rename cancel");
				break;
	    }
		gtk_widget_destroy (dialog);
		if (result == GTK_RESPONSE_CANCEL) break;
	}

	g_list_free (base);
	refresh_list (curr_view);
	if (new_list != NULL)
	{
		gtk_clist_unselect_all (GTK_CLIST(curr_view->clist));
		for (tmp = new_list; tmp != NULL; tmp = tmp->next)
			select_row_by_filename (curr_view, (gchar *) tmp->data);
		e2_list_free_with_data (&new_list);
	}
	postpare_dialog (FALSE, NULL);
}

void sync_dirs_cb (GtkWidget *widget)
{
	e2_pane_change_dir (app.pane_other, app.pane_current->path);
}

void view_cb (GtkWidget *widget)
{
	FileInfo *info;
	GList *tmp, *base;

	disable_refresh();
	base = tmp = get_selection(curr_view);

	if (!e2_option_bool_get ("use-external-viewer"))
	{
		for (; tmp != NULL; tmp = tmp->next)
		{
			info = tmp->data;
			e2_view_dialog_show(info->filename);
		}
	} else
	{
		info = tmp->data;
		view_file (info->filename);
	}

	g_list_free(base);
	reenable_refresh();
	gtk_widget_grab_focus(curr_view->clist);
}

void toggle_tag_cb ()
{
	GList *tmp = GTK_CLIST(curr_view->clist)->selection;
	FileInfo *info;
	gint row;

	gtk_signal_emit_by_name(GTK_OBJECT(curr_view->clist), "end-selection");

	for (; tmp != NULL; tmp = tmp->next)
	{
		row = (gint)tmp->data;
		info = gtk_clist_get_row_data(GTK_CLIST(curr_view->clist), row);
		if (g_list_find(curr_view->tagged, info) != NULL)
		{
			curr_view->tagged = g_list_remove(curr_view->tagged, info);
			gtk_clist_set_background(GTK_CLIST(curr_view->clist), row, &CLIST_COLOR);
		}
		else
		{
			curr_view->tagged = g_list_append(curr_view->tagged, info);
			gtk_clist_set_background(GTK_CLIST(curr_view->clist), row, e2_option_color_get ("color-tag"));
		}
	}
	focus_row(curr_view, curr_view->row+1, TRUE, TRUE, TRUE);
}

void refresh_cb (GtkWidget *widget)
{
	e2_widget_set_cursor (GDK_WATCH);
	disable_refresh();
	/* Since the refresh_list will switch the curr_view and other_view... */
	refresh_list(other_view);
	refresh_list(other_view);
	reenable_refresh();
	e2_widget_set_cursor (GDK_LEFT_PTR);
}

gboolean configure_cb (GtkWidget *widget, gchar *page)
{
	create_config_dialog (page);
	return TRUE;
}

void quit_cb (GtkWidget *widget)
{
	printd (DEBUG, "callback: quit");
	printd (DEBUG, "writing other config files");
	write_filetypes_file();
//	write_keys_file();
	write_plugins_file();
	write_config_file ();
	printd (DEBUG, "writing config file");
	e2_option_write_to_file ();
	printd (DEBUG, "writing cache file");
	e2_cache_write_to_file ();
	printd (DEBUG, "touch config dir");
	touch_config_dir();
	printd (DEBUG, "clean options");
	destroy_config ();
	printd (DEBUG, "clean bookmarks");
	e2_bookmark_clean ();
	printd (DEBUG, "clean actions");
	e2_actions_clean ();
	printd (DEBUG, "gtk_main_quit ()");
	gtk_main_quit ();
	printd (DEBUG, "exit (0)");
	exit (0);
}

void find_cb (GtkWidget *widget)
{
	gchar *s, *pattern;
	gint i, last;

	gtk_widget_set_sensitive(app.main_window, FALSE);
	disable_refresh();
	create_user_prompt(_("Enter a filename or partial filename to find:"),
			               curr_view->last_find, TRUE, &s);
	gtk_main();

	if (s != NULL)
	{
		strncpy(curr_view->last_find, s, sizeof(curr_view->last_find));
		pattern = g_strdup_printf("*%s*", s);
		if (GTK_CLIST(curr_view->clist)->selection != NULL)
		{
			last = (gint)GTK_CLIST(curr_view->clist)->selection->data;
			if (last == (GTK_CLIST(curr_view->clist)->rows - 1))
			  i = 0;
			else
			  i = last + 1;
		}
		else
		{
			i = 0;
			last = GTK_CLIST(curr_view->clist)->rows - 1;
			if (last < 0)
			{
			  g_free(pattern);
			  g_free(s);
			  reenable_refresh();
			  gtk_widget_set_sensitive(app.main_window, TRUE);
			  return;
			}
		}

		while (i != last)
		{
			FileInfo *info = gtk_clist_get_row_data(GTK_CLIST(curr_view->clist), i);
			if (g_pattern_match_simple(pattern, info->filename))
			{
			  focus_row(curr_view, i, TRUE, TRUE, TRUE);
			  break;
			}
			i++;
			if (i >= GTK_CLIST(curr_view->clist)->rows)
			  i = 0;
		}
		if (i == last)
			focus_row(curr_view, last, TRUE, FALSE, TRUE);
		g_free(pattern);
		g_free(s);
	}
	else
		focus_row(curr_view, curr_view->row, FALSE, FALSE, TRUE);

	reenable_refresh();
	gtk_widget_set_sensitive(app.main_window, TRUE);
	gtk_widget_grab_focus (curr_view->clist);
}

void open_cb (GtkWidget *widget)
{
	printd (DEBUG, "callback: open");
	FileInfo *info;
	gchar *ext, *action, *sep;

	disable_refresh();
	if ((info = get_first_selected(curr_view)) == NULL)
	{
		reenable_refresh();
		return;
	}

	if (is_dir (info))
		ext = ".<directory>";
	else if (is_executable (info))
		ext = ".<executable>";
	else
		ext = strchr(info->filename, '.');

	if (ext == NULL) /* no extension */
	{
		if (S_ISEXE(info->statbuf.st_mode))
		{
			gchar command_line[NAME_MAX+2];
			g_snprintf(command_line, sizeof(command_line), "./%s", info->filename);
			e2_command_run (command_line, E2_COMMAND_RANGE_DEFAULT);
		}
		else if (is_text(info->filename))
		{
			if (e2_option_bool_get ("use-internal-viewer"))
				e2_view_dialog_show(info->filename);
			else
				view_file(info->filename);
		}
		else
		{
			create_open_query_dialog(info->filename);
		}
		reenable_refresh();
		WAIT_FOR_EVENTS;
		return;
	}

	if (ext == info->filename) /* its a dot file */
	{
		if (is_text(info->filename))
		{
			if (e2_option_bool_get ("use-internal-viewer"))
			  e2_view_dialog_show(info->filename);
			else
			  view_file(info->filename);
		}
		else
		{
			create_open_query_dialog(info->filename);
		}
		reenable_refresh();
		WAIT_FOR_EVENTS;
		return;
	}

	/* the file has an extension */
	do
	{
		ext++; /* get rid of the . prefix */
		action = get_default_action_for_ext(ext);
		if (action != NULL)
		{
			if ((sep = strchr(action, '@')) != NULL)
			  exec_filetype_action(sep+1);
			else
			  exec_filetype_action(action);
			g_free(action);
			reenable_refresh();
			gtk_widget_grab_focus (curr_view->clist);
			WAIT_FOR_EVENTS;
			return;
		}
	} while ((ext = strchr(ext, '.')) != NULL);

	/* Didn't find any matching extensions */
	WAIT_FOR_EVENTS;
	gtk_widget_set_sensitive(app.main_window, FALSE);
	create_init_filetype_dialog (info);
	reenable_refresh();
}

void open_with_cb (GtkWidget *widget)
{
	gchar *command;

	gtk_widget_set_sensitive(app.main_window, FALSE);
	create_user_prompt(_("Enter command:"), "", FALSE, &command);
	gtk_main();

	gtk_widget_set_sensitive(app.main_window, TRUE);
	gtk_widget_grab_focus(curr_view->clist);
	if (command == NULL) return;

	if (strchr(command, '%') == NULL)
	{
		GString *command_line = g_string_new(command);
		GList *base, *tmp;

		disable_refresh();
		base = tmp = get_selection(curr_view);
		for (; tmp != NULL; tmp = tmp->next)
		{
			FileInfo *info = tmp->data;
			g_string_append_printf(command_line , " \"%s\"", info->filename);
		}

		e2_command_run (command_line->str, E2_COMMAND_RANGE_DEFAULT);
		g_string_free(command_line, TRUE);
		reenable_refresh();
	}
	else
		e2_command_run (command, E2_COMMAND_RANGE_DEFAULT);

	g_free(command);
}

void open_in_other_pane_cb (GtkWidget *widget)
{
	gchar *path;
	FileInfo *info;
	
	disable_refresh();
	info = gtk_clist_get_row_data(GTK_CLIST(curr_view->clist), curr_view->row);
	path = g_strdup_printf("%s/%s", curr_view->dir, info->filename);
	e2_pane_change_dir (app.pane_other, path);
	g_free(path);
	focus_row(other_view, other_view->row, FALSE, FALSE, TRUE);
	reenable_refresh();
}
