/* $Id: e2_output.c,v 1.26 2004/07/23 18:02:38 cairo Exp $
 * 
 * Copyright (C) 2004 Florian Zähringer <flo.zaehringer@web.de>
 * Copyright (C) 2003-2004 tooar <tooar@gmx.net>
 * Copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>
#include <glob.h>
#include <string.h>
#include <errno.h>
#include <pwd.h>
#include <sys/types.h>
//FIXME: bad trick, what's the right way to do it?
#define GTK_TEXT_USE_INTERNAL_UNSUPPORTED_API
#include <gtk/gtktextlayout.h>
#undef GTK_TEXT_USE_INTERNAL_UNSUPPORTED_API
#include "emelfm2.h"
#include "e2_output.h"
#include "e2_output_help.h"
#include "debug.h"
#include "e2_option.h"
#include "e2_command.h"
#include "e2_command_line.h"
#include "e2_utils.h"
#include "e2_context_menu.h"

#define E2_PANED_TOLERANCE 10

  /*****************/
 /***** utils *****/
/*****************/

//stolen from gtktextview.c
static gboolean clamp_iter_onscreen (GtkTextView *text_view, GtkTextIter *iter)
{
	GdkRectangle visible_rect;
	gtk_text_view_get_visible_rect (text_view, &visible_rect);

	return gtk_text_layout_clamp_iter_to_vrange (text_view->layout, iter,
		visible_rect.y, visible_rect.y + visible_rect.height);
}

  /*******************/
 /***** actions *****/
/*******************/

static void print_action (gpointer data, gchar *message)
{
	gchar *real_message = e2_utils_str_replace (message, "\\n", "\n");
	e2_output_print (real_message, NULL, FALSE, FALSE, FALSE, NULL);
	g_free (real_message);
}

static void help_action (gpointer data, gchar *arg)
{
	if ((arg != NULL) && (*arg != '\0') && (g_str_equal ("keys", arg)))
		e2_output_help_key_bindings ();
	else
		e2_output_help_command_line ();
}

static void scroll_helper (gboolean down, gchar *arg, gboolean page)
{
	printd (DEBUG, "scroll_helper (down:%d,arg:%s,page:%d)", down, arg, page);
	E2_OutputRuntime *rt = &app.output;
	g_return_if_fail (rt->scroll != NULL);
	gchar *end = NULL;
	gdouble times = 1.0;
	if (arg != NULL)
		times = g_ascii_strtod (arg, &end);
	if (end == arg)
		times = 1.0;
	GtkAdjustment *vadj = gtk_scrolled_window_get_vadjustment 
		(GTK_SCROLLED_WINDOW (rt->scroll));
	gdouble value = gtk_adjustment_get_value (vadj);
	gint inc = page ? vadj->page_increment : vadj->step_increment;
	if (down)
	{
		value += (inc * times);
		if (value > (vadj->upper - vadj->page_size))
			value = vadj->upper - vadj->page_size;
	}
	else
		value -= (inc * times);

	gtk_adjustment_set_value (vadj, value);
}

static void scroll_step (gboolean down, gchar *arg)
{
	scroll_helper (down, arg, FALSE);
}

static void scroll_page (gboolean down, gchar *arg)
{
	scroll_helper (down, arg, TRUE);
}
static void scroll_all (gboolean down, gchar *arg)
{
	E2_OutputRuntime *rt = &app.output;
	g_return_if_fail (rt->scroll != NULL);
	GtkAdjustment *vadj = gtk_scrolled_window_get_vadjustment 
		(GTK_SCROLLED_WINDOW (rt->scroll));
	if (down)
		gtk_adjustment_set_value (vadj, vadj->upper - vadj->page_size);
	else
		gtk_adjustment_set_value (vadj, 0.0);
}

void e2_output_clear ()
{
	GtkTextTagTable *table = gtk_text_buffer_get_tag_table (app.output.buffer);
	GtkTextBuffer *buffer = gtk_text_buffer_new (table);
	GtkTextIter start;
	gtk_text_buffer_get_start_iter (buffer, &start);
	app.output.mark = gtk_text_buffer_create_mark (buffer,
		"internal-end-mark", &start, FALSE);
	
	gtk_text_view_set_buffer (app.output.text, buffer);
	app.output.buffer = gtk_text_view_get_buffer (app.output.text);
	g_object_unref (buffer);
	app.output.onscreen = TRUE;
}

/**
 * @brief print @a msg to output pane
 *
 * print @a msg to output pane.
 *
 * @param msg       actual message
 * @param origin    context to which the message belongs or NULL for default
 * @param error     is this an error message? if yes style parameters are ignored
 * @param debug     is this a debug message?
 * @param newline   should this message be printed on a separate line?
 * @param first_tag one or more tags to apply terminated by NULL
 *
 * @return 
 **/
void e2_output_print (gchar *msg, gchar *origin, gboolean error,
	gboolean debug, gboolean newline, const gchar *first_tag, ...)
{
	//early exit
	if (msg == NULL) return;
	
	if (error)
	{
		e2_output_print (msg, origin, FALSE, debug, newline, "red", "bold", NULL);
		return;
	}
//	printd (DEBUG, "e2_ouput_print (msg:,origin:%s,error:%d,debug:%d,newline:%d,first_tag:)",
//		origin, error, debug, newline);

	//if this is not a debug message show the output pane
	//if the user wishes that
	if ((!debug) && (e2_option_bool_get ("show-output-window-on-output")))
		e2_output_show_pane (NULL);

	//get the text buffer from the output pane
	GtkTextBuffer *buffer = app.output.buffer;

	gint len = strlen (msg);

	//FIXME: handle \r correctly
	//save if there's a line feed '\r' in the message
	gboolean line_back = FALSE;
	//delete how many characters in the last message of this origin
	//(used to handle backspaces)
	gint back = 0;
	//backspace
	while (((gint) msg[0]) == '\b')
	{
		back++;
		msg++;
		len--;
	}
	
	//true if there are backspaces not at the beginning of the message
	gboolean backspaces_left = FALSE;
	//message variable should be freed?
	gboolean free_message = FALSE;
	gint i;
	//handle control characters
	for (i = 0; i < len; i++)
	{
//		printd (DEBUG, "%c - %d", msg[i], (gint) msg[i]);
		//escape
		if (((int) msg[i]) == 27)
		{
			//clear sequence: ESC[2J
			if (((i + 3) < len) && (msg[i + 1] == '[') &&
				(msg[i + 2] == '2') && (msg[i + 3] == 'J'))
			{
				e2_output_clear ();
				return;
			}
			
		}
		//line feed
		else if (((gint) msg[i]) == '\r')
			line_back = TRUE;
		//backspace
		else if (((gint) msg[i]) == '\b')
		{
			//yes, there are backspaces in the middle
			backspaces_left = TRUE;
			gint j = 1;
			//find a character back in the string that is not
			//already a backspace
			while ((j <= len) && (msg[i - j] == '\b'))
				j++;
			//and overwrite it with backspace
			if (j <= i)
				msg[i - j] = '\b';
			//if there is no non-backspace character delete one 
			//from the previous message
			else
				back++;
		}
	}
	
	if (backspaces_left)
	{
		free_message = TRUE;
		//FIXME: is there a faster way to do that?
		//replace backspaces with nothing
		msg = e2_utils_str_replace (msg, "\b", NULL);
	}

	//assure that there's an origin set
	if (origin == NULL)
		origin = "default";

	//name of the mark that saves the end of the origin's output
	gchar *mark_end_name = g_strconcat (origin, "-end", NULL);
	//is this origin new
	gboolean is_new = FALSE;
	GtkTextIter start, end;
	//try to get insert mark
	//it exists if there has been output in this origin before
	GtkTextMark *mark = gtk_text_buffer_get_mark (buffer, origin);
	//if not, create it at the end
	if (mark == NULL)
	{
		is_new = TRUE;
		gtk_text_buffer_get_end_iter (buffer, &end);
		mark = gtk_text_buffer_create_mark (buffer, origin, &end, TRUE);
	} else
	{
		//the default origin always outputs to the end of the textview
		//(ie the output is not "glued together"
		if (g_str_equal (origin, "default"))
		{
			gtk_text_buffer_get_end_iter(buffer, &end);
			gtk_text_buffer_move_mark (buffer, mark, &end);
		}
		else
			gtk_text_buffer_get_iter_at_mark (buffer, &end, mark);
		
	}
	//move the scroll helper mark
	//(it is used by the scroll callback to find out if the user is
	//following output)
	gtk_text_buffer_move_mark (buffer, app.output.mark, &end);

	gtk_text_buffer_place_cursor (buffer, &end);

	//do we have to overwrite a previous message's characters in front of us?
	//(usually if a previous message had a line feed '\r' in it)
	GtkTextMark *mark_del = gtk_text_buffer_get_mark (buffer, mark_end_name);
	if (mark_del != NULL)
	{
		GtkTextIter iter_del;
		gtk_text_buffer_get_iter_at_mark (buffer, &iter_del, mark_del);
		gtk_text_buffer_delete (buffer, &end, &iter_del);
	
	}
	//do we have to delete a previous message's characters behind us?
	//(usually from backspaces in the message)
	if (back > 0)
	{
			gint offset = gtk_text_iter_get_offset (&end);
			GtkTextIter backi;
			gtk_text_buffer_get_iter_at_offset (buffer, &backi, offset - back);
			gtk_text_buffer_delete (buffer, &backi, &end);
			gtk_text_buffer_get_iter_at_offset (buffer, &end, offset - back);
	}
	//delete the delete mark, it's only used one time
	if (mark_del != NULL)
		gtk_text_buffer_delete_mark (buffer, mark_del);

	//find out if it is necessary to insert a newline character before
	//the message because it is of a different origin than the last one
	//and the last one hasn't printed one yet;
	//save the origin of the last message
	static gchar *origin_buffer = NULL;
	if (origin_buffer != origin)
	{
		if ((!gtk_text_iter_starts_line (&end)) &&
			(is_new || (g_str_equal (origin, "default"))))
		{
			gtk_text_buffer_insert  (buffer, &end, "\n", 1);
			GtkTextMark *mark_end = gtk_text_buffer_get_insert (buffer);
			gtk_text_buffer_get_iter_at_mark (buffer, &end, mark_end);
		}
	}

	if (newline && (!gtk_text_iter_starts_line (&end)))
	{
		printd (DEBUG, "inserted additional newline because the message requested it");
		gtk_text_buffer_insert  (buffer, &end, "\n", 1);
		GtkTextMark *mark_end = gtk_text_buffer_get_insert (buffer);
		gtk_text_buffer_get_iter_at_mark (buffer, &end, mark_end);
	}
	origin_buffer = origin;
	
	//FIXME: actually we should check this much earlier and use only utf8
	//functions internally
	//check if message is already utf8
	if (g_utf8_validate (msg, -1, NULL))
		gtk_text_buffer_insert  (buffer, &end, msg, -1);
	else
	{
		//convert to utf and actually insert it into the textbuffer
		gchar *utf = STR2UTF (msg);
		gtk_text_buffer_insert  (buffer, &end, utf, -1);
		g_free (utf);
	}
	if (free_message)
		g_free (msg);

	//get end of last insert
	GtkTextMark *mark_end = gtk_text_buffer_get_insert (buffer);
	gtk_text_buffer_get_iter_at_mark (buffer, &end, mark_end);
	if (newline)
	{
		gtk_text_buffer_insert  (buffer, &end, "\n", 1);
		mark_end = gtk_text_buffer_get_insert (buffer);
		gtk_text_buffer_get_iter_at_mark (buffer, &end, mark_end);
	}
	gtk_text_buffer_get_iter_at_mark (buffer, &start, mark);
	
	//if there was a line feed, save a delete mark for the next message
	if ((line_back) && (!is_new))
		gtk_text_buffer_create_mark (buffer, mark_end_name, &end, TRUE);
	else
		gtk_text_buffer_move_mark (buffer, mark, &end);

	g_free (mark_end_name);

	//scroll down if necessary
	if (_e2_option_bool_get (app.output.opt_jump) && 
		(app.output.onscreen || !_e2_option_bool_get (app.output.opt_jump_follow)))
	{
		GtkTextIter iter;
		gtk_text_buffer_get_iter_at_mark (buffer, &iter, mark_end);
		if ((gtk_text_iter_is_end (&iter)) ||
			!_e2_option_bool_get (app.output.opt_jump_end))
		{
			if (clamp_iter_onscreen (app.output.text, &iter))
				gtk_text_view_scroll_to_mark (app.output.text, mark_end,
					0.0, TRUE, 0.1, 1.0);
		}
	}

	if (first_tag == NULL)
		return;
		
	//process tag arguments for different message styles
	va_list args;
	va_start (args, first_tag);
	const gchar *tag_name;
	tag_name = first_tag;
	while (tag_name)
	{
		GtkTextTag *tag = gtk_text_tag_table_lookup (buffer->tag_table,
			tag_name);

		if (tag == NULL)
		{
			g_warning ("%s: no tag with name '%s'!", G_STRLOC, tag_name);
			tag_name = va_arg (args, const gchar*);
			continue;
		}

		gtk_text_buffer_apply_tag (buffer, tag, &start, &end);

		tag_name = va_arg (args, const gchar*);
	}
	va_end (args);
}

  /*********************/
 /***** callbacks *****/
/*********************/

static void output_pane_toggle_cb (GtkWidget *widget)
{
	e2_action_run_simple ("output.adjust ratio", "*,100%");
}

static void output_pane_fs_cb (GtkWidget *widget)
{
	e2_action_run_simple ("output.adjust ratio", "*,0");
}

gboolean e2_output_show_pane (GtkWidget *widget)
{
	if (!GTK_WIDGET_VISIBLE (app.output.text))
		e2_action_run_simple ("output.adjust ratio", "*");
	return FALSE;
}

gboolean e2_output_hide_pane (GtkWidget *widget)
{
	if (GTK_WIDGET_VISIBLE (app.output.text))
		e2_action_run_simple ("output.adjust ratio", "100%");
	return FALSE;
}

static void edit_op_cb (GtkWidget *menuitem, gchar *option)
{
	create_config_dialog (option);
}

static void toggle_op_bool_cb (GtkWidget *menuitem, gchar *option)
{
	e2_option_bool_toggle (option);
}

static void set_op_wrap_cb (GtkWidget *menuitem, gchar *option)
{
	if (!gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menuitem)))
		return;
	GSList *group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menuitem));
	gint index = 2 - g_slist_index (group, menuitem);
	if (e2_option_int_get (option) != index)
	{
		e2_option_sel_set (option, index);
		gtk_text_view_set_wrap_mode (app.output.text, index); 
	}
}

static void show_context_menu_cb (GtkWidget *widget, GdkEventButton *event)
{
	GtkWidget *menu, *submenu ,*menuitem;
	menu = gtk_menu_new ();
	menuitem = add_menu_item2 (menu, "_edit config", GTK_STOCK_PROPERTIES,
		edit_op_cb, "output");
	e2_menu_add_separator (menu);
	menuitem = add_menu_item2 (menu, "show/hide output", NULL, 
			output_pane_toggle_cb, NULL);
	menuitem = add_menu_item2 (menu, "output fullscreen", NULL, 
			output_pane_fs_cb, NULL);
	menuitem = add_menu_item2 (menu, "clear output", NULL, 
			e2_output_clear, NULL);
	e2_menu_add_separator (menu);
	submenu = add_submenu2 (menu, "wrap mode");
	GSList *group = NULL;
	menuitem = add_menu_radio_button (submenu, &group, "none",
		(e2_option_int_get ("output-wrap-mode") == 0),  
		set_op_wrap_cb, "output-wrap-mode");
	menuitem = add_menu_radio_button (submenu, &group, "everywhere",
		(e2_option_int_get ("output-wrap-mode") == 1), 
		set_op_wrap_cb, "output-wrap-mode");
	menuitem = add_menu_radio_button (submenu, &group, "words",
		(e2_option_int_get ("output-wrap-mode") == 2),  
		set_op_wrap_cb, "output-wrap-mode");
	//FIXME: margin missing
	e2_menu_add_separator (menu);
	menuitem = e2_menu_add_check (menu, "show on new message",
		e2_option_bool_get ("show-output-window-on-output"), 
		toggle_op_bool_cb, "show-output-window-on-output");
/*	menuitem = e2_menu_add_check (menu, "show on focus",
		e2_option_bool_get ("show-output-window-on-focus-in"), 
		toggle_op_bool_cb, "show-output-window-on-focus-in");
	menuitem = e2_menu_add_check (menu, "hide on unfocus",
		e2_option_bool_get ("hide-output-window-on-focus-out"), 
		toggle_op_bool_cb, "hide-output-window-on-focus-out"); */
	gboolean opt = e2_option_bool_get ("output-jump-new");
	menuitem = e2_menu_add_check (menu, "scroll to output",
		opt, toggle_op_bool_cb, "output-jump-new");
	menuitem = e2_menu_add_check (menu, "only scroll when following",
		e2_option_bool_get ("output-jump-new-following"), 
		toggle_op_bool_cb, "output-jump-new-following");
	if (!opt)
		gtk_widget_set_sensitive (menuitem, FALSE);
	menuitem = e2_menu_add_check (menu, "only scroll to end",
		e2_option_bool_get ("output-jump-new-end"), 
		toggle_op_bool_cb, "output-jump-new-end");
	if (!opt)
		gtk_widget_set_sensitive (menuitem, FALSE);
	e2_menu_popup (menu, event->button, event->time);
	
}

static gboolean output_text_clicked_cb (GtkWidget *widget, GdkEventButton *event)
{
	if (event->button == 1)
	{
		e2_action_run_simple ("command line: focus", NULL);
		return FALSE;
	}
	else if (event->button == 2)
	{
		e2_output_hide_pane (NULL);
		return TRUE;
	}
	else if (event->button == 3)
	{
		show_context_menu_cb (widget, event);
		return TRUE;
	}

	return FALSE;
}

static void scrolled_cb (GtkAdjustment *adjust, E2_OutputRuntime *rt)
{
	GtkTextIter iter;
	if (_e2_option_bool_get (app.output.opt_jump_end))
		gtk_text_buffer_get_end_iter (rt->buffer, &iter);
	else
		gtk_text_buffer_get_iter_at_mark (rt->buffer, &iter, rt->mark);

	if (clamp_iter_onscreen (rt->text, &iter))
	{
		if (rt->onscreen)
			printd (NOTICE, "not following anymore");
		rt->onscreen = FALSE;
	} else
	{
		if (!rt->onscreen)
			printd (NOTICE, "following again");
		rt->onscreen = TRUE;
	}
}

  /******************/
 /***** public *****/
/******************/

void e2_output_update_style (E2_OutputRuntime *rt)
{
	gtk_text_view_set_wrap_mode (rt->text,
		e2_option_int_get ("output-wrap-mode")); 
	gtk_text_view_set_left_margin (rt->text,
		e2_option_int_get ("output-left-margin"));
	gtk_text_view_set_right_margin (rt->text,
		e2_option_int_get ("output-right-margin"));
	//set font
	PangoFontDescription *font_desc = pango_font_description_from_string
		(e2_option_str_get ("output-font"));
	rt->font_size = pango_font_description_get_size (font_desc);
	GtkTextTagTable *table = gtk_text_buffer_get_tag_table (rt->buffer);
	GtkTextTag *tag = gtk_text_tag_table_lookup (table, "small");
	if (tag != NULL)
	{
		g_object_set (G_OBJECT (tag), "size",
			(gint) (PANGO_SCALE_SMALL * rt->font_size), NULL);
	} else
	{
		gtk_text_buffer_create_tag (rt->buffer, "small", "size",
			(gint) (PANGO_SCALE_SMALL * rt->font_size), NULL);
	}
	gtk_widget_modify_font (GTK_WIDGET (rt->text), font_desc);
	pango_font_description_free (font_desc);
	tag = gtk_text_tag_table_lookup (table, "red");
	if (tag != NULL)
	{
		g_object_set (G_OBJECT (tag), "foreground",
			e2_option_str_get ("color-negative"), NULL);
	} 
	tag = gtk_text_tag_table_lookup (table, "grey");
	if (tag != NULL)
	{
		g_object_set (G_OBJECT (tag), "foreground",
			e2_option_str_get ("color-unimportant"), NULL);
	} 
}

/*
static gboolean test_cb (GtkWidget *widget, GdkEventMotion *event)
{
	int x, y;
	GdkModifierType state;
	if (event->is_hint)
	{
	        gdk_window_get_pointer(event->window, &x, &y, &state);
	        printf("x = %d y = %d\n", x, y);
	}
	else 
	{
	        printf("x: %lf y: %lf\n", event->x, event->y);
	}                
	return FALSE;  
} 
*/
/**
 * @brief create output pane
 *
 * create output pane with textview and buffer and some basic tags.
 *
 * @param rt runtime struct to work on
 *
 * @return scrolled window with a textview
 **/
GtkWidget *e2_output_create (E2_OutputRuntime *rt)
{
	rt->onscreen = TRUE;
	rt->mark = NULL;
	
	//create scrolled window
	rt->scroll = e2_widget_get_sw
		(GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (rt->scroll),
		GTK_SHADOW_OUT);

	rt->text = GTK_TEXT_VIEW (gtk_text_view_new ());
	gtk_container_add (GTK_CONTAINER (rt->scroll), GTK_WIDGET (rt->text));
	gtk_text_view_set_editable (rt->text, FALSE);
	gtk_text_view_set_cursor_visible (rt->text, FALSE);
	GTK_WIDGET_UNSET_FLAGS (rt->text, GTK_CAN_FOCUS);
	
//	g_signal_connect (G_OBJECT (rt->text), "motion-notify-event",
//		G_CALLBACK (test_cb), NULL);
	gtk_widget_set_events (GTK_WIDGET (rt->text),
		gtk_widget_get_events (GTK_WIDGET (rt->text))
		| GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK);
//	g_signal_connect (G_OBJECT (rt->text), "size-allocate",
//		G_CALLBACK (output_text_adjusted_cb), NULL);
	g_signal_connect (G_OBJECT (rt->text), "button_press_event",
		G_CALLBACK (output_text_clicked_cb), NULL);
	g_signal_connect_after (G_OBJECT (rt->text->vadjustment), "value-changed",
		G_CALLBACK (scrolled_cb), rt);
	
	gtk_widget_show (GTK_WIDGET (rt->text));
	
	rt->buffer = gtk_text_view_get_buffer (rt->text);

	e2_output_update_style (rt);

	//create some default output tags
	gtk_text_buffer_create_tag (rt->buffer, "blue", "foreground", "blue", NULL);
	gtk_text_buffer_create_tag (rt->buffer, "red", "foreground",
		e2_option_str_get ("color-negative"), NULL);
	gtk_text_buffer_create_tag (rt->buffer, "grey", "foreground", 
		e2_option_str_get ("color-unimportant"), NULL);
	gtk_text_buffer_create_tag (rt->buffer, "bold", "weight", PANGO_WEIGHT_BOLD, NULL);
	gtk_text_buffer_create_tag (rt->buffer, "ul", "underline", PANGO_UNDERLINE_SINGLE, NULL);
//	gtk_text_buffer_create_tag (rt->buffer, "small", "size", , NULL);
	gtk_text_buffer_create_tag (rt->buffer, "link", "foreground", "blue",
		"underline", PANGO_UNDERLINE_NONE, NULL);
	gtk_text_buffer_create_tag (rt->buffer, "link-active", "foreground", "blue",
		"underline", PANGO_UNDERLINE_SINGLE, NULL);

	GtkTextIter end;
	gtk_text_buffer_get_end_iter (rt->buffer, &end);
	rt->mark = gtk_text_buffer_create_mark (rt->buffer,
		"internal-end-mark", &end, FALSE);
	
	rt->opt_jump = e2_option_get ("output-jump-new");
	rt->opt_jump_follow = e2_option_get ("output-jump-new-following");
	rt->opt_jump_end = e2_option_get ("output-jump-new-end");

	return rt->scroll;
}

void e2_output_init ()
{
	e2_option_bool_register ("show-output-window-on-output", "command.output:options",
		"show output window when a new message appears",
		"this will assure you don't miss any messages", NULL, FALSE);
	e2_option_bool_register ("output-jump-new", "command.output:options",
		"scroll to new output", "automatically scroll to the place "
		"where new messages have appeared", NULL, TRUE);
	e2_option_bool_register ("output-jump-new-following",
		"command.output:options", "only scroll when really following",
		"don't scroll to new output if you've manually scrolled away",
		"output-jump-new", TRUE);
	e2_option_bool_register ("output-jump-new-end",
		"command.output:options", "only scroll when new output is at the end",
		"don't scroll to new output if it's not at the end",
		"output-jump-new", TRUE);
	gchar *opt_wrap_mode[] = {"none", "everywhere", "words", NULL};
	e2_option_sel_register ("output-wrap-mode", "command.output:style",
		"line wrap mode", "the wrap mode 'none' means that you'll get a "
		"horizontal scrollbar. 'words' will only break the line between words",
		NULL, 2, opt_wrap_mode);
	e2_option_int_register ("output-left-margin", "command.output:style",
		"left text margin (pixel)", "left margin in pixels of text "
		"in the output pane", NULL, 6, 0, 1000);
	e2_option_int_register ("output-right-margin", "command.output:style",
		"right text margin (pixel)", "right margin in pixels of text "
		"in the output pane", NULL, 2, 0, 1000);

	e2_action_register ("output.clear", E2_ACTION_TYPE_ITEM,
		e2_output_clear, NULL, FALSE);
	e2_action_register ("output.print", E2_ACTION_TYPE_ITEM,
		print_action, NULL, TRUE);
	e2_action_register ("output.help", E2_ACTION_TYPE_ITEM,
		help_action, NULL, TRUE);
	e2_action_register ("output.scroll up", E2_ACTION_TYPE_ITEM,
		scroll_step, GINT_TO_POINTER (FALSE), TRUE);
	e2_action_register ("output.scroll down", E2_ACTION_TYPE_ITEM,
		scroll_step, GINT_TO_POINTER (TRUE), TRUE);
	e2_action_register ("output.page up", E2_ACTION_TYPE_ITEM,
		scroll_page, GINT_TO_POINTER (FALSE), TRUE);
	e2_action_register ("output.page down", E2_ACTION_TYPE_ITEM,
		scroll_page, GINT_TO_POINTER (TRUE), TRUE);
	e2_action_register ("output.goto top", E2_ACTION_TYPE_ITEM,
		scroll_all, GINT_TO_POINTER (FALSE), TRUE);
	e2_action_register ("output.goto bottom", E2_ACTION_TYPE_ITEM,
		scroll_all, GINT_TO_POINTER (TRUE), TRUE);
}
